@@ -22,9 +22,7 @@
 
 #include <glib-object.h>
 #include <glib/gprintf.h>
-#if HAVE_GIO
 #include <gio/gio.h>
-#endif
 
 #include "gegl.h"
 #include "gegl-types-internal.h"
@@ -37,12 +35,7 @@
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
 #include "gegl-sampler-lanczos.h"
-#include "gegl-sampler-downsharp.h"
-#include "gegl-sampler-downsize.h"
-#include "gegl-sampler-downsmooth.h"
-#include "gegl-sampler-upsharp.h"
-#include "gegl-sampler-upsize.h"
-#include "gegl-sampler-upsmooth.h"
+#include "gegl-sampler-lohalo.h"
 #include "gegl-buffer-index.h"
 #include "gegl-tile-backend.h"
 #include "gegl-buffer-iterator.h"
@@ -56,7 +49,7 @@ gegl_buffer_pixel_set (GeglBuffer *buffer,
                        guchar     *buf)
 {
   gint  tile_width  = buffer->tile_storage->tile_width;
-  gint  tile_height  = buffer->tile_storage->tile_width;
+  gint  tile_height = buffer->tile_storage->tile_width;
   gint  px_size     = gegl_buffer_px_size (buffer);
   gint  bpx_size    = babl_format_get_bytes_per_pixel (format);
   Babl *fish        = NULL;
@@ -73,43 +66,41 @@ gegl_buffer_pixel_set (GeglBuffer *buffer,
       fish = babl_fish (buffer->format, format);
     }
 
-  {
-    if (!(buffer_y + y >= buffer_abyss_y &&
-          buffer_y + y < abyss_y_total &&
-          buffer_x + x >= buffer_abyss_x &&
-          buffer_x + x < abyss_x_total))
-      { /* in abyss */
-        return;
-      }
-    else
-      {
-        gint      tiledy = buffer_y + buffer->shift_y + y;
-        gint      tiledx = buffer_x + buffer->shift_x + x;
+  if (!(buffer_y + y >= buffer_abyss_y &&
+        buffer_y + y < abyss_y_total &&
+        buffer_x + x >= buffer_abyss_x &&
+        buffer_x + x < abyss_x_total))
+    { /* in abyss */
+      return;
+    }
+  else
+    {
+      gint      tiledy = buffer_y + buffer->shift_y + y;
+      gint      tiledx = buffer_x + buffer->shift_x + x;
 
-        GeglTile *tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
-                                                gegl_tile_indice (tiledx, tile_width),
-                                                gegl_tile_indice (tiledy, tile_height),
-                                                0);
+      GeglTile *tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
+                                                  gegl_tile_indice (tiledx, tile_width),
+                                                  gegl_tile_indice (tiledy, tile_height),
+                                                  0);
 
-        if (tile)
-          {
-            gint    offsetx = gegl_tile_offset (tiledx, tile_width);
-            gint    offsety = gegl_tile_offset (tiledy, tile_height);
-            guchar *tp;
+      if (tile)
+        {
+          gint    offsetx = gegl_tile_offset (tiledx, tile_width);
+          gint    offsety = gegl_tile_offset (tiledy, tile_height);
+          guchar *tp;
 
-            gegl_tile_lock (tile);
-            tp = gegl_tile_get_data (tile) +
-                 (offsety * tile_width + offsetx) * px_size;
-            if (fish)
-              babl_process (fish, buf, tp, 1);
-            else
-              memcpy (tp, buf, bpx_size);
+          gegl_tile_lock (tile);
+          tp = gegl_tile_get_data (tile) +
+            (offsety * tile_width + offsetx) * px_size;
+          if (fish)
+            babl_process (fish, buf, tp, 1);
+          else
+            memcpy (tp, buf, bpx_size);
 
-            gegl_tile_unlock (tile);
-            gegl_tile_unref (tile);
-          }
-      }
-  }
+          gegl_tile_unlock (tile);
+          gegl_tile_unref (tile);
+        }
+    }
   return;
 }
 #endif
@@ -126,7 +117,6 @@ gegl_buffer_in_abyss( GeglBuffer *buffer,
   gint  abyss_x_total  = buffer_abyss_x + buffer->abyss.width;
   gint  abyss_y_total  = buffer_abyss_y + buffer->abyss.height;
 
-
   gint tiledy = y + buffer_shift_y;
   gint tiledx = x + buffer_shift_x;
 
@@ -349,8 +339,7 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
   gint  buffer_abyss_y = buffer->abyss.y + buffer_shift_y;
   gint  abyss_x_total  = buffer_abyss_x + buffer->abyss.width;
   gint  abyss_y_total  = buffer_abyss_y + buffer->abyss.height;
-  gint  i;
-  gint  factor = 1;
+  gint  factor         = 1<<level;
 
   /* roi specified, override buffers extent */
   if (roi)
@@ -361,11 +350,6 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
       buffer_y = roi->y + buffer_shift_y;
     }
 
-  for (i = 0; i < level; i++)
-    {
-      factor *= 2;
-    }
-
   buffer_abyss_x /= factor;
   buffer_abyss_y /= factor;
   abyss_x_total  /= factor;
@@ -402,8 +386,6 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
       gint tiledy  = buffer_y + bufy;
       gint offsety = gegl_tile_offset (tiledy, tile_height);
 
-
-
       gint bufx    = 0;
 
       if (!(buffer_y + bufy + (tile_height) >= buffer_abyss_y &&
@@ -604,9 +586,7 @@ gegl_buffer_set_unlocked (GeglBuffer          *buffer,
     }
   else
 #endif
-    {
-      gegl_buffer_iterate (buffer, rect, src, rowstride, TRUE, format, 0);
-    }
+  gegl_buffer_iterate (buffer, rect, src, rowstride, TRUE, format, 0);
 
   if (gegl_buffer_is_shared(buffer))
     {
@@ -629,8 +609,6 @@ gegl_buffer_set (GeglBuffer          *buffer,
 }
 
 
-
-
 #if 0
 
 /*
@@ -975,6 +953,8 @@ gegl_buffer_get_unlocked (GeglBuffer          *buffer,
       gegl_buffer_iterate (buffer, NULL, dest_buf, rowstride, FALSE, format, 0);
       return;
     }
+
+  g_return_if_fail (rect);
   if (rect->width == 0 ||
       rect->height == 0)
     {
@@ -1024,7 +1004,7 @@ gegl_buffer_get_unlocked (GeglBuffer          *buffer,
       sample_buf = g_malloc (buf_width * buf_height * bpp);
       gegl_buffer_iterate (buffer, &sample_rect, sample_buf, GEGL_AUTO_ROWSTRIDE, FALSE, format, level);
 #if 1
-  /* slows testing of rendering code speed to much for now and
+  /* slows testing of rendering code speed too much for now and
    * no time to make a fast implementation
    */
 
@@ -1086,14 +1066,17 @@ gegl_buffer_get_abyss (GeglBuffer *buffer)
   return &buffer->abyss;
 }
 
+GType
+gegl_sampler_gtype_from_enum (GeglSamplerType sampler_type);
+
 void
 gegl_buffer_sample (GeglBuffer       *buffer,
                     gdouble           x,
                     gdouble           y,
-                    gdouble           scale,
+                    GeglMatrix2      *scale,
                     gpointer          dest,
                     const Babl       *format,
-                    GeglInterpolation interpolation)
+                    GeglSamplerType   sampler_type)
 {
   GType desired_type;
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
@@ -1104,11 +1087,11 @@ gegl_buffer_sample (GeglBuffer       *buffer,
   return;
 #endif
 
-  desired_type = gegl_sampler_type_from_interpolation (interpolation);
+  desired_type = gegl_sampler_gtype_from_enum (sampler_type);
 
   /* unset the cached sampler if it dosn't match the needs */
   if (buffer->sampler != NULL &&
-     (!G_TYPE_CHECK_INSTANCE_TYPE (buffer->sampler, desired_type) || 
+     (!G_TYPE_CHECK_INSTANCE_TYPE (buffer->sampler, desired_type) ||
        buffer->sampler_format != format
       ))
     {
@@ -1126,9 +1109,8 @@ gegl_buffer_sample (GeglBuffer       *buffer,
       buffer->sampler_format = format;
       gegl_sampler_prepare (buffer->sampler);
     }
-  gegl_sampler_get (buffer->sampler, x, y, dest);
 
-  /* if (scale < 1.0) do decimation, possibly using pyramid instead */
+  gegl_sampler_get (buffer->sampler, x, y, scale, dest);
 }
 
 void
@@ -1143,7 +1125,6 @@ gegl_buffer_sample_cleanup (GeglBuffer *buffer)
     }
 }
 
-
 void
 gegl_buffer_copy (GeglBuffer          *src,
                   const GeglRectangle *src_rect,
@@ -1224,18 +1205,3 @@ gegl_buffer_dup (GeglBuffer *buffer)
   return new_buffer;
 }
 
-void
-gegl_buffer_sampler (GeglBuffer       *buffer,
-                    gdouble           x,
-                    gdouble           y,
-                    gdouble           scale,
-                    gpointer          dest,
-                    const Babl       *format,
-                    gpointer          sampler2)
-{
-  GeglSampler *sampler = sampler2;
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
-  g_return_if_fail (GEGL_IS_SAMPLER (sampler));
-
-  gegl_sampler_get (sampler, x, y, dest);
-}
