@@ -64,18 +64,20 @@ typedef struct GeglBufferIterators
 {
   /* current region of interest */
   gint          length;             /* length of current data in pixels */
-  gpointer      data[GEGL_BUFFER_MAX_ITERATORS]; 
-  GeglRectangle roi[GEGL_BUFFER_MAX_ITERATORS];                 
+  gpointer      data[GEGL_BUFFER_MAX_ITERATORS];
+  GeglRectangle roi[GEGL_BUFFER_MAX_ITERATORS]; /* roi of the current data */
 
   /* the following is private: */
   gint           iterators;
   gint           iteration_no;
-  GeglRectangle  rect       [GEGL_BUFFER_MAX_ITERATORS];
-  const Babl    *format     [GEGL_BUFFER_MAX_ITERATORS];
-  GeglBuffer    *buffer     [GEGL_BUFFER_MAX_ITERATORS];
+  gboolean       is_finished;
+  GeglRectangle  rect       [GEGL_BUFFER_MAX_ITERATORS]; /* the region we iterate on. They can be different from
+                                                            each other, but width and height are the same */
+  const Babl    *format     [GEGL_BUFFER_MAX_ITERATORS]; /* The format required for the data */
+  GeglBuffer    *buffer     [GEGL_BUFFER_MAX_ITERATORS]; /* currently a subbuffer of the original, need to go away */
   guint          flags      [GEGL_BUFFER_MAX_ITERATORS];
-  gpointer       buf        [GEGL_BUFFER_MAX_ITERATORS]; 
-  GeglBufferTileIterator   i[GEGL_BUFFER_MAX_ITERATORS]; 
+  gpointer       buf        [GEGL_BUFFER_MAX_ITERATORS]; /* no idea */
+  GeglBufferTileIterator   i[GEGL_BUFFER_MAX_ITERATORS];
 } GeglBufferIterators;
 
 
@@ -95,7 +97,7 @@ static gboolean gegl_buffer_scan_compatible (GeglBuffer *bufferA,
                                              GeglBuffer *bufferB,
                                              gint        xB,
                                              gint        yB)
-{ 
+{
   if (bufferA->tile_storage->tile_width !=
       bufferB->tile_storage->tile_width)
     return FALSE;
@@ -141,8 +143,8 @@ gegl_buffer_tile_iterator_next (GeglBufferTileIterator *i)
   gint  tile_height    = buffer->tile_storage->tile_height;
   gint  buffer_shift_x = buffer->shift_x;
   gint  buffer_shift_y = buffer->shift_y;
-  gint  buffer_x       = buffer->extent.x + buffer_shift_x;
-  gint  buffer_y       = buffer->extent.y + buffer_shift_y;
+  gint  buffer_x       = i->roi.x + buffer_shift_x;
+  gint  buffer_y       = i->roi.y + buffer_shift_y;
 
   if (i->roi.width == 0 || i->roi.height == 0)
     return FALSE;
@@ -264,8 +266,8 @@ gegl_buffer_iterator_add (GeglBufferIterator  *iterator,
     roi = self==0?&(buffer->extent):&(i->rect[0]);
   i->rect[self]=*roi;
 
-  /* XXX: if this buffer creation could be avoided, it would be a speedup */
-  i->buffer[self]=gegl_buffer_create_sub_buffer (buffer, roi);
+  i->buffer[self]= g_object_ref (buffer);
+
   if (format)
     i->format[self]=format;
   else
@@ -292,7 +294,7 @@ gegl_buffer_iterator_add (GeglBufferIterator  *iterator,
     }
 
   i->buf[self] = NULL;
-  
+
   if (i->format[self] == i->buffer[self]->format)
     {
       i->flags[self] |= GEGL_BUFFER_FORMAT_COMPATIBLE;
@@ -374,7 +376,7 @@ gboolean gegl_buffer_iterator_next     (GeglBufferIterator *iterator)
   gboolean result = FALSE;
   gint no;
 
-  if (i->buf[0] == (void*)0xdeadbeef)
+  if (i->is_finished)
     g_error ("%s called on finished buffer iterator", G_STRFUNC);
   if (i->iteration_no == 0)
     {
@@ -445,11 +447,11 @@ gboolean gegl_buffer_iterator_next     (GeglBufferIterator *iterator)
           if (res != result)
             {
               g_print ("%i==%i != 0==%i\n", no, res, result);
-             } 
+            }
           g_assert (res == result);
 
-          if ((i->flags[no] & GEGL_BUFFER_FORMAT_COMPATIBLE) && 
-              i->roi[no].width == i->i[no].buffer->tile_storage->tile_width 
+          if ((i->flags[no] & GEGL_BUFFER_FORMAT_COMPATIBLE) &&
+              i->roi[no].width == i->i[no].buffer->tile_storage->tile_width
            )
             {
               /* direct access */
@@ -474,7 +476,7 @@ gboolean gegl_buffer_iterator_next     (GeglBufferIterator *iterator)
             }
         }
       else
-        { 
+        {
           /* we copy the roi from iterator 0  */
           i->roi[no] = i->roi[0];
           i->roi[no].x += (i->rect[no].x-i->rect[0].x);
@@ -524,7 +526,7 @@ gboolean gegl_buffer_iterator_next     (GeglBufferIterator *iterator)
       g_print ("%f %f\n", (100.0*direct_read/(in_direct_read+direct_read)),
                            100.0*direct_write/(in_direct_write+direct_write));
 #endif
-      i->buf[0]=(void*)0xdeadbeef;
+      i->is_finished = TRUE;
       g_slice_free (GeglBufferIterators, i);
     }
 
@@ -533,11 +535,13 @@ gboolean gegl_buffer_iterator_next     (GeglBufferIterator *iterator)
 }
 
 GeglBufferIterator *gegl_buffer_iterator_new (GeglBuffer          *buffer,
-                                              const GeglRectangle *roi, 
+                                              const GeglRectangle *roi,
                                               const Babl          *format,
                                               guint                flags)
 {
   GeglBufferIterator *i = (gpointer)g_slice_new0 (GeglBufferIterators);
+  /* Because the iterator is nulled above, we can forgo explicitly setting
+   * i->is_finished to FALSE. */
   gegl_buffer_iterator_add (i, buffer, roi, format, flags);
   return i;
 }
