@@ -22,7 +22,6 @@
 
 #include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
-#include "gegl-sampler.h"
 #include "gegl-tile-handler.h"
 #include "gegl-buffer-iterator.h"
 
@@ -122,8 +121,6 @@ GeglBuffer *
 gegl_buffer_new_ram (const GeglRectangle *extent,
                      const Babl          *format);
 
-GType gegl_sampler_type_from_interpolation (GeglInterpolation interpolation);
-
 void            gegl_buffer_sampler           (GeglBuffer     *buffer,
                                                gdouble         x,
                                                gdouble         y,
@@ -166,9 +163,15 @@ struct _GeglTile
   GeglTile        *next_shared;
   GeglTile        *prev_shared;
 
-  void (*destroy_notify) (gpointer pixels,
-                          gpointer data);
-  gpointer         destroy_notify_data;
+  /* called when the tile is about to be destroyed */
+  GeglDestroyNotify destroy_notify;
+  gpointer          destroy_notify_data;
+
+  /* called when the tile has been unlocked which typically means tile
+   * data has changed
+   */
+  GeglTileCallback unlock_notify;
+  gpointer         unlock_notify_data;
 };
 
 #ifndef __GEGL_TILE_C
