@@ -42,9 +42,7 @@
 #include <glib-object.h>
 #include <glib/gstdio.h>
 #include <glib/gprintf.h>
-#if HAVE_GIO
 #include <gio/gio.h>
-#endif
 
 #include "gegl.h"
 #include "gegl-types-internal.h"
@@ -64,12 +62,7 @@
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
 #include "gegl-sampler-lanczos.h"
-#include "gegl-sampler-downsharp.h"
-#include "gegl-sampler-downsize.h"
-#include "gegl-sampler-downsmooth.h"
-#include "gegl-sampler-upsharp.h"
-#include "gegl-sampler-upsize.h"
-#include "gegl-sampler-upsmooth.h"
+#include "gegl-sampler-lohalo.h"
 #include "gegl-types-internal.h"
 #include "gegl-utils.h"
 #include "gegl-id-pool.h"
@@ -363,6 +356,8 @@ gint gegl_buffer_leaks (void)
         g_printerr ("%s\n", buffer->alloc_stack_trace);
       }
   }
+  g_list_free (allocated_buffers_list);
+  allocated_buffers_list = NULL;
 #endif
 
   return allocated_buffers - de_allocated_buffers;
@@ -402,9 +397,11 @@ static void
 gegl_buffer_finalize (GObject *object)
 {
 #ifdef GEGL_BUFFER_DEBUG_ALLOCATIONS
+  g_free (GEGL_BUFFER (object)->alloc_stack_trace);
   allocated_buffers_list = g_list_remove (allocated_buffers_list, object);
 #endif
 
+  g_free (GEGL_BUFFER (object)->path);
   de_allocated_buffers++;
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -464,8 +461,6 @@ gegl_buffer_constructor (GType                  type,
   GeglTileBackend *backend;
   GeglTileHandler *handler;
   GeglTileSource  *source;
-  gint             tile_width;
-  gint             tile_height;
 
   gint width;
   gint height;
@@ -611,9 +606,6 @@ gegl_buffer_constructor (GType                  type,
 
   g_assert (backend);
 
-  tile_width  = backend->priv->tile_width;
-  tile_height = backend->priv->tile_height;
-
   if (buffer->extent.width == -1 &&
       buffer->extent.height == -1) /* no specified extents,
                                       inheriting from source */
@@ -902,7 +894,7 @@ gegl_buffer_get_alloc_stack (void)
   char **function_names = NULL;
   int    i              = 0;
   int    result_size    = 0;
-  
+
   n_functions = backtrace (functions, MAX_N_FUNCTIONS);
   function_names = backtrace_symbols (functions, n_functions);
 
@@ -1173,7 +1165,7 @@ gegl_tile_storage_new_cached (gint tile_width, gint tile_height,
       item->tile_height = tile_height;
       item->babl_fmt = babl_fmt;
 
-      if (use_ram || 
+      if (use_ram ||
           !gegl_config()->swap ||
           g_str_equal (gegl_config()->swap, "RAM") ||
           g_str_equal (gegl_config()->swap, "ram"))
