@@ -21,13 +21,15 @@
 
 #include <glib-object.h>
 #include <babl/babl.h>
+#include <gegl-matrix.h>
 
 G_BEGIN_DECLS
 
 #define GEGL_TYPE_BUFFER (gegl_buffer_get_type ())
 #define GEGL_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_BUFFER, GeglBuffer))
 #ifndef __GEGL_BUFFER_TYPES_H__
-typedef struct _GeglBuffer     GeglBuffer;
+typedef struct _GeglBuffer   GeglBuffer;
+typedef struct _GeglSampler  GeglSampler;
 #endif
 
 /***
@@ -54,10 +56,25 @@ GType           gegl_buffer_get_type          (void) G_GNUC_CONST;
  * possible to pass in NULL for both extent and format, a NULL extent creates
  * an empty buffer and a NULL format makes the buffer default to "RGBA float".
  */
-GeglBuffer*     gegl_buffer_new               (const GeglRectangle *extent,
+GeglBuffer *    gegl_buffer_new               (const GeglRectangle *extent,
                                                const Babl          *format);
 
 
+
+/**
+ * gegl_buffer_new_for_backend:
+ * @extent: the geometry of the buffer (origin, width and height) a
+ * GeglRectangle.
+ * @backend: an instance of a GeglTileBackend subclass.
+ *
+ * Create a new GeglBuffer from a backend, if NULL is passed in the extent of
+ * the buffer will be inherited from the extent of the backend.
+ *
+ * returns a GeglBuffer, that holds a reference to the provided backend.
+ */
+GeglBuffer * gegl_buffer_new_for_backend (const GeglRectangle *extent,
+                                          void                *backend);
+
 /**
  * gegl_buffer_open:
  * @path: the path to a gegl buffer on disk.
@@ -68,7 +85,7 @@ GeglBuffer*     gegl_buffer_new               (const GeglRectangle *extent,
  *
  * Returns: a GeglBuffer object.
  */
-GeglBuffer*     gegl_buffer_open              (const gchar         *path);
+GeglBuffer *    gegl_buffer_open              (const gchar         *path);
 
 /**
  * gegl_buffer_save:
@@ -93,7 +110,7 @@ void            gegl_buffer_save              (GeglBuffer          *buffer,
  *
  * Returns: a #GeglBuffer object.
  */
-GeglBuffer     *gegl_buffer_load              (const gchar         *path);
+GeglBuffer *     gegl_buffer_load              (const gchar         *path);
 
 /**
  * gegl_buffer_flush:
@@ -112,7 +129,7 @@ void            gegl_buffer_flush             (GeglBuffer          *buffer);
  *
  * Create a new sub GeglBuffer, that is a view on a larger buffer.
  */
-GeglBuffer*     gegl_buffer_create_sub_buffer (GeglBuffer          *buffer,
+GeglBuffer *    gegl_buffer_create_sub_buffer (GeglBuffer          *buffer,
                                                const GeglRectangle *extent);
 
 /**
@@ -246,9 +263,9 @@ void            gegl_buffer_set               (GeglBuffer          *buffer,
  * Get the native babl format of the buffer.
  *
  * Returns: the babl format used for storing pixels in the buffer.
- * 
+ *
  */
-const Babl    *gegl_buffer_get_format        (GeglBuffer           *buffer);
+const Babl *    gegl_buffer_get_format        (GeglBuffer           *buffer);
 
 /**
  * gegl_buffer_clear:
@@ -290,46 +307,36 @@ void            gegl_buffer_copy              (GeglBuffer          *src,
  * lead to a buffer that shares the raster data with the original on a tile
  * by tile COW basis. This is not yet implemented
  */
-GeglBuffer    * gegl_buffer_dup               (GeglBuffer       *buffer);
+GeglBuffer *    gegl_buffer_dup               (GeglBuffer       *buffer);
 
-typedef enum {
-  GEGL_INTERPOLATION_NEAREST = 0,
-  GEGL_INTERPOLATION_LINEAR,
-  GEGL_INTERPOLATION_CUBIC,
-  GEGL_INTERPOLATION_LANCZOS,
-  GEGL_INTERPOLATION_DOWNSHARP,
-  GEGL_INTERPOLATION_DOWNSIZE,
-  GEGL_INTERPOLATION_DOWNSMOOTH,
-  GEGL_INTERPOLATION_UPSHARP,
-  GEGL_INTERPOLATION_UPSIZE,
-  GEGL_INTERPOLATION_UPSMOOTH
-} GeglInterpolation;
 
 /**
  * gegl_buffer_sample:
  * @buffer: the GeglBuffer to sample from
  * @x: x coordinate to sample in buffer coordinates
  * @y: y coordinate to sample in buffer coordinates
- * @scale: the scale we're fetching at (<1.0 can lead to decimation)
+ * @scale: a matrix that indicates scaling factors, see
+ * gegl_sampler_compute_scale the same.
  * @dest: buffer capable of storing one pixel in @format.
  * @format: the format to store the sampled color in.
- * @interpolation: the interpolation behavior to use, currently only nearest
- * neighbour is implemented for this API, bilinear, bicubic and lanczos needs
- * to be ported from working code. Valid values: GEGL_INTERPOLATION_NEAREST and
- * GEGL_INTERPOLATION_LINEAR, GEGL_INTERPOLATON_CUBIC and
- * GEGL_INTERPOLATION_LANCZOS.
+ * @sampler_type: the sampler type to use,
+ * to be ported from working code. Valid values: GEGL_SAMPLER_NEAREST,
+ * GEGL_SAMPLER_LINEAR, GEGL_SAMPLER_CUBIC,
+ * GEGL_SAMPLER_LANCZOS and GEGL_SAMPLER_LOHALO
  *
  * Query interpolate pixel values at a given coordinate using a specified form
  * of interpolation. The samplers used cache for a small neighbourhood of the
  * buffer for more efficient access.
  */
-void            gegl_buffer_sample            (GeglBuffer       *buffer,
-                                               gdouble           x,
-                                               gdouble           y,
-                                               gdouble           scale,
-                                               gpointer          dest,
-                                               const Babl       *format,
-                                               GeglInterpolation interpolation);
+void gegl_buffer_sample (GeglBuffer       *buffer,
+                         gdouble           x,
+                         gdouble           y,
+                         GeglMatrix2      *scale,
+                         gpointer          dest,
+                         const Babl       *format,
+                         GeglSamplerType   sampler_type);
+
+
 
 
 /**
@@ -343,16 +350,56 @@ void            gegl_buffer_sample            (GeglBuffer       *buffer,
  */
 void            gegl_buffer_sample_cleanup    (GeglBuffer *buffer);
 
+
+
 /**
- * gegl_buffer_interpolation_from_string:
+ * gegl_sampler_type_from_string:
  * @string: the string to look up
  *
  * Looks up the GeglInterpolation corresponding to a string, if no matching
- * interpolation is found returns GEGL_INTERPOLATION_NEAREST.
+ * interpolation is found GEGL_SAMPLER_NEAREST is returned.
  */
-GeglInterpolation gegl_buffer_interpolation_from_string (const gchar *string);
+GeglSamplerType gegl_sampler_type_from_string (const gchar *string);
 
+/**
+ * gegl_buffer_sampler_new:
+ * @buffer: buffer to create a new sampler for
+ * @format: format we want data back in
+ * @sampler_type: the sampler type to use,
+ * to be ported from working code. Valid values: GEGL_SAMPLER_NEAREST,
+ * GEGL_SAMPLER_LINEAR, GEGL_SAMPLER_CUBIC,
+ * GEGL_SAMPLER_LANCZOS and GEGL_SAMPLER_LOHALO
+ *
+ * Create a new sampler, when you are done with the sampler, g_object_unref
+ * it.
+ */
+GeglSampler * gegl_buffer_sampler_new (GeglBuffer       *buffer,
+                                       Babl             *format,
+                                       GeglSamplerType   sampler_type);
 
+/**
+ * gegl_sampler_get:
+ * @sampler: a GeglSampler gotten from gegl_buffer_sampler_new
+ * @x: x coordinate to sample
+ * @y: y coordinate to sample
+ * @scale: matrix representing extent of sampling area in source buffer.
+ * @output: memory location for output data.
+ *
+ * Perform a sampling with the provided @sampler.
+ */
+void  gegl_sampler_get         (GeglSampler *sampler,
+                                gdouble      x,
+                                gdouble      y,
+                                GeglMatrix2 *scale,
+                                void        *output);
+
+/**
+ * gegl_sampler_get_context_rect:
+ * @sampler: a GeglSampler gotten from gegl_buffer_sampler_new
+ *
+ * Returns:The context rectangle of the given @sampler.
+ */
+const GeglRectangle * gegl_sampler_get_context_rect (GeglSampler *sampler);
 
 /**
  * gegl_buffer_linear_new:
@@ -365,7 +412,7 @@ GeglInterpolation gegl_buffer_interpolation_from_string (const gchar *string);
  *
  * Returns: a GeglBuffer that can be used as any other GeglBuffer.
  */
-GeglBuffer *gegl_buffer_linear_new           (const GeglRectangle *extent,
+GeglBuffer * gegl_buffer_linear_new          (const GeglRectangle *extent,
                                               const Babl          *format);
 
 /**
@@ -382,7 +429,7 @@ GeglBuffer *gegl_buffer_linear_new           (const GeglRectangle *extent,
  *
  * Creates a GeglBuffer backed by a linear memory buffer that already exists,
  * of the given @extent in the specified @format. babl_format ("R'G'B'A u8")
- * for instance to make a normal 8bit buffer. 
+ * for instance to make a normal 8bit buffer.
  *
  * Returns: a GeglBuffer that can be used as any other GeglBuffer.
  */
@@ -406,7 +453,7 @@ GeglBuffer * gegl_buffer_linear_new_from_data (const gpointer       data,
  * request is compatible with the underlying data storage direct access
  * to the underlying data is provided.
  */
-gpointer       *gegl_buffer_linear_open      (GeglBuffer          *buffer,
+gpointer *      gegl_buffer_linear_open      (GeglBuffer          *buffer,
                                               const GeglRectangle *extent,
                                               gint                *rowstride,
                                               const Babl          *format);
@@ -432,11 +479,9 @@ void            gegl_buffer_linear_close     (GeglBuffer    *buffer,
  * Return the abyss extent of a buffer, this expands out to the parents extent in
  * subbuffers.
  */
-const GeglRectangle* gegl_buffer_get_abyss  (GeglBuffer           *buffer);
+const GeglRectangle * gegl_buffer_get_abyss  (GeglBuffer           *buffer);
 
 #include <gegl-buffer-iterator.h>
 
-/**
- */
 G_END_DECLS
 #endif
