@@ -36,9 +36,11 @@ enum
   PROP_LAST
 };
 
+static void      gegl_sampler_cubic_finalize (GObject      *gobject);
 static void      gegl_sampler_cubic_get (GeglSampler  *sampler,
                                          gdouble       x,
                                          gdouble       y,
+                                         GeglMatrix2  *scale,
                                          void         *output);
 static void      get_property           (GObject      *gobject,
                                          guint         prop_id,
@@ -63,6 +65,7 @@ gegl_sampler_cubic_class_init (GeglSamplerCubicClass *klass)
 
   object_class->set_property = set_property;
   object_class->get_property = get_property;
+  object_class->finalize     = gegl_sampler_cubic_finalize;
 
   sampler_class->get     = gegl_sampler_cubic_get;
 
@@ -93,13 +96,20 @@ gegl_sampler_cubic_class_init (GeglSamplerCubicClass *klass)
 
 }
 
+static void
+gegl_sampler_cubic_finalize (GObject *object)
+{
+  g_free (GEGL_SAMPLER_CUBIC (object)->type);
+  G_OBJECT_CLASS (gegl_sampler_cubic_parent_class)->finalize (object);
+}
+
 static void
 gegl_sampler_cubic_init (GeglSamplerCubic *self)
 {
- GEGL_SAMPLER (self)->context_rect.x = -1;
- GEGL_SAMPLER (self)->context_rect.y = -1;
- GEGL_SAMPLER (self)->context_rect.width = 4;
- GEGL_SAMPLER (self)->context_rect.height = 4;
+ GEGL_SAMPLER (self)->context_rect[0].x = -1;
+ GEGL_SAMPLER (self)->context_rect[0].y = -1;
+ GEGL_SAMPLER (self)->context_rect[0].width = 4;
+ GEGL_SAMPLER (self)->context_rect[0].height = 4;
  GEGL_SAMPLER (self)->interpolate_format = babl_format ("RaGaBaA float");
  self->b=1.0;
  self->c=0.0;
@@ -126,6 +136,7 @@ void
 gegl_sampler_cubic_get (GeglSampler *self,
                         gdouble      x,
                         gdouble      y,
+                        GeglMatrix2 *scale,
                         void        *output)
 {
   GeglSamplerCubic *cubic = (GeglSamplerCubic*)(self);
@@ -148,7 +159,7 @@ gegl_sampler_cubic_get (GeglSampler *self,
   gint              dx,dy;
   gint              i;
 
-  context_rect = self->context_rect;
+  context_rect = self->context_rect[0];
   dx = (gint) x;
   dy = (gint) y;
   sampler_bptr = gegl_sampler_get_ptr (self, dx, dy);
@@ -184,9 +195,7 @@ gegl_sampler_cubic_get (GeglSampler *self,
            }
      }
 
-  babl_process (babl_fish (self->interpolate_format, self->format),
-                newval, output, 1);
-
+  babl_process (self->fish, newval, output, 1);
 }
 
 static void
