@@ -46,6 +46,7 @@ static void           lanczos_lookup (GeglSamplerLanczos *sampler);
 static void           gegl_sampler_lanczos_get (GeglSampler  *sampler,
                                                 gdouble       x,
                                                 gdouble       y,
+                                                GeglMatrix2  *scale,
                                                 void         *output);
 static void           get_property             (GObject      *gobject,
                                                 guint         prop_id,
@@ -145,10 +146,11 @@ void
 gegl_sampler_lanczos_get (GeglSampler *self,
                           gdouble      x,
                           gdouble      y,
+                          GeglMatrix2 *scale,
                           void        *output)
 {
   GeglSamplerLanczos      *lanczos      = GEGL_SAMPLER_LANCZOS (self);
-  GeglRectangle            context_rect = self->context_rect;
+  GeglRectangle            context_rect = self->context_rect[0];
   gfloat                  *sampler_bptr;
   gdouble                  x_sum, y_sum;
   gfloat                   newval[4] = {0.0, 0.0, 0.0, 0.0};
@@ -238,10 +240,10 @@ set_property (GObject      *object,
       case PROP_LANCZOS_WIDTH:
         {
         self->lanczos_width = g_value_get_int (value);
-        GEGL_SAMPLER (self)->context_rect.x = - self->lanczos_width;
-        GEGL_SAMPLER (self)->context_rect.y = - self->lanczos_width;
-        GEGL_SAMPLER (self)->context_rect.width = self->lanczos_width*2+1;
-        GEGL_SAMPLER (self)->context_rect.height = self->lanczos_width*2+1;
+        GEGL_SAMPLER (self)->context_rect[0].x = - self->lanczos_width;
+        GEGL_SAMPLER (self)->context_rect[0].y = - self->lanczos_width;
+        GEGL_SAMPLER (self)->context_rect[0].width = self->lanczos_width*2+1;
+        GEGL_SAMPLER (self)->context_rect[0].height = self->lanczos_width*2+1;
         }
         break;
 
