@@ -15,11 +15,14 @@
  * <http://www.gnu.org/licenses/>.
  *
  * 2007 © Øyvind Kolås
+ * 2009 © Nicolas Robidoux
+ * 2011 © Adam Turcotte
  */
 #include "config.h"
 
 #include <glib-object.h>
 #include <string.h>
+#include <math.h>
 
 #include "gegl.h"
 #include "gegl-types-internal.h"
@@ -31,12 +34,7 @@
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
 #include "gegl-sampler-lanczos.h"
-#include "gegl-sampler-downsharp.h"
-#include "gegl-sampler-downsize.h"
-#include "gegl-sampler-downsmooth.h"
-#include "gegl-sampler-upsharp.h"
-#include "gegl-sampler-upsize.h"
-#include "gegl-sampler-upsmooth.h"
+#include "gegl-sampler-lohalo.h"
 
 enum
 {
@@ -44,7 +42,6 @@ enum
   PROP_BUFFER,
   PROP_FORMAT,
   PROP_CONTEXT_RECT,
-  PROP_INVERSE_JACOBIAN,
   PROP_LAST
 };
 
@@ -102,45 +99,30 @@ gegl_sampler_class_init (GeglSamplerClass *klass)
                                         "Input pad, for image buffer input.",
                                         GEGL_TYPE_BUFFER,
                                         G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
-
-  g_object_class_install_property (
-                 object_class,
-                 PROP_INVERSE_JACOBIAN,
-                 g_param_spec_pointer ("inverse_jacobian",
-                                       "Inverse Jacobian",
-                                       "Inverse Jacobian matrix, for certain samplers.",
-                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 static void
 gegl_sampler_init (GeglSampler *self)
 {
-  GeglRectangle context_rect = {0,0,1,1};
-  GeglRectangle sampler_rectangle = {0,0,0,0};
-  self->sampler_buffer = NULL;
+  int i;
   self->buffer = NULL;
-  self->context_rect = context_rect;
-  self->sampler_rectangle = sampler_rectangle;
+  for (i=0; i<GEGL_SAMPLER_MIPMAP_LEVELS; ++i) {
+    GeglRectangle context_rect = {0,0,1,1};
+    GeglRectangle sampler_rectangle = {0,0,0,0};
+    self->sampler_buffer[i] = NULL;
+    self->context_rect[i] = context_rect;
+    self->sampler_rectangle[i] = sampler_rectangle;
+  }
 }
 
 void
 gegl_sampler_get (GeglSampler *self,
                   gdouble      x,
                   gdouble      y,
+                  GeglMatrix2 *scale,
                   void        *output)
 {
-  GeglSamplerClass *klass;
-
-#if 0  /* avoiding expensive typecheck here */
-  g_return_if_fail (GEGL_IS_SAMPLER (self));
-#endif
-
-  self->x = x;
-  self->y = y;
-
-  klass = GEGL_SAMPLER_GET_CLASS (self);
-
-  klass->get (self, x, y, output);
+  self->get (self, x, y, scale, output);
 }
 
 void
@@ -162,8 +144,8 @@ gegl_sampler_prepare (GeglSampler *self)
    * This makes the cache rect invalid, in case the data in the buffer
    * has changed:
    */
-  self->sampler_rectangle.width = 0;
-  self->sampler_rectangle.height = 0;
+  self->sampler_rectangle[0].width = 0;
+  self->sampler_rectangle[0].height = 0;
 
 #if 0
   if (self->cache_buffer) /* Force a regetting of the region, even
@@ -173,7 +155,7 @@ gegl_sampler_prepare (GeglSampler *self)
       self->cache_buffer = NULL;
     }
 #endif
-
+  self->get = klass->get; /* cache the sampler in the instance */
 }
 
 void
@@ -192,11 +174,15 @@ gegl_sampler_set_buffer (GeglSampler *self, GeglBuffer *buffer)
 static void
 finalize (GObject *gobject)
 {
+  int i;
   GeglSampler *sampler = GEGL_SAMPLER (gobject);
-  if (sampler->sampler_buffer)
+  for (i=0; i<GEGL_SAMPLER_MIPMAP_LEVELS; ++i)
     {
-      g_free (sampler->sampler_buffer);
-      sampler->sampler_buffer = NULL;
+      if (sampler->sampler_buffer[i])
+        {
+          g_free (sampler->sampler_buffer[i]);
+          sampler->sampler_buffer[i] = NULL;
+        }
     }
   G_OBJECT_CLASS (gegl_sampler_parent_class)->finalize (gobject);
 }
@@ -236,24 +222,24 @@ gegl_sampler_get_ptr (GeglSampler *const sampler,
    * in GEGL).
    */
   const gint maximum_width_and_height = 64;
-  g_assert (sampler->context_rect.width  <= maximum_width_and_height);
-  g_assert (sampler->context_rect.height <= maximum_width_and_height);
+  g_assert (sampler->context_rect[0].width  <= maximum_width_and_height);
+  g_assert (sampler->context_rect[0].height <= maximum_width_and_height);
 
-  if (( sampler->sampler_buffer == NULL )
+  if (( sampler->sampler_buffer[0] == NULL )
       ||
-      ( x + sampler->context_rect.x < sampler->sampler_rectangle.x )
+      ( x + sampler->context_rect[0].x < sampler->sampler_rectangle[0].x )
       ||
-      ( y + sampler->context_rect.y < sampler->sampler_rectangle.y )
+      ( y + sampler->context_rect[0].y < sampler->sampler_rectangle[0].y )
       ||
-      ( x + sampler->context_rect.x + sampler->context_rect.width
-        >= sampler->sampler_rectangle.x + sampler->sampler_rectangle.width )
+      ( x + sampler->context_rect[0].x + sampler->context_rect[0].width
+        > sampler->sampler_rectangle[0].x + sampler->sampler_rectangle[0].width )
       ||
-      ( y + sampler->context_rect.y + sampler->context_rect.height
-        >= sampler->sampler_rectangle.y + sampler->sampler_rectangle.height ))
+      ( y + sampler->context_rect[0].y + sampler->context_rect[0].height
+        > sampler->sampler_rectangle[0].y + sampler->sampler_rectangle[0].height ))
     {
       /*
        * fetch_rectangle will become the value of
-       * sampler->sampler_rectangle:
+       * sampler->sampler_rectangle[0]:
        */
       GeglRectangle fetch_rectangle;
 
@@ -271,21 +257,21 @@ gegl_sampler_get_ptr (GeglSampler *const sampler,
        * much if at all, as should be.
        */
       fetch_rectangle.x =
-        x + sampler->context_rect.x
-        - ( maximum_width_and_height - sampler->context_rect.width  ) / 8;
+        x + sampler->context_rect[0].x
+        - ( maximum_width_and_height - sampler->context_rect[0].width  ) / 8;
       fetch_rectangle.y =
-        y + sampler->context_rect.y
-        - ( maximum_width_and_height - sampler->context_rect.height ) / 8;
+        y + sampler->context_rect[0].y
+        - ( maximum_width_and_height - sampler->context_rect[0].height ) / 8;
 
       fetch_rectangle.width  = maximum_width_and_height;
       fetch_rectangle.height = maximum_width_and_height;
 
-      if (sampler->sampler_buffer == NULL)
+      if (sampler->sampler_buffer[0] == NULL)
         {
           /*
            * Always request the same amount of pixels:
            */
-          sampler->sampler_buffer =
+          sampler->sampler_buffer[0] =
             g_malloc0 (( maximum_width_and_height * maximum_width_and_height )
                        * bpp);
         }
@@ -294,16 +280,16 @@ gegl_sampler_get_ptr (GeglSampler *const sampler,
                        1.0,
                        &fetch_rectangle,
                        sampler->interpolate_format,
-                       sampler->sampler_buffer,
+                       sampler->sampler_buffer[0],
                        GEGL_AUTO_ROWSTRIDE);
 
-      sampler->sampler_rectangle = fetch_rectangle;
+      sampler->sampler_rectangle[0] = fetch_rectangle;
     }
 
-  dx = x - sampler->sampler_rectangle.x;
-  dy = y - sampler->sampler_rectangle.y;
-  buffer_ptr = (guchar *)sampler->sampler_buffer;
-  sof = ( dx + dy * sampler->sampler_rectangle.width ) * bpp;
+  dx = x - sampler->sampler_rectangle[0].x;
+  dy = y - sampler->sampler_rectangle[0].y;
+  buffer_ptr = (guchar *)sampler->sampler_buffer[0];
+  sof = ( dx + dy * sampler->sampler_rectangle[0].width ) * bpp;
 
   return (gfloat*)(buffer_ptr+sof);
 }
@@ -327,18 +313,18 @@ gegl_sampler_get_from_buffer (GeglSampler *const sampler,
    * in GEGL).
    */
   const gint maximum_width_and_height = 64;
-  g_assert (sampler->context_rect.width  <= maximum_width_and_height);
-  g_assert (sampler->context_rect.height <= maximum_width_and_height);
+  g_assert (sampler->context_rect[0].width  <= maximum_width_and_height);
+  g_assert (sampler->context_rect[0].height <= maximum_width_and_height);
 
-  if (( sampler->sampler_buffer == NULL )
+  if (( sampler->sampler_buffer[0] == NULL )
       ||
-      ( x < sampler->sampler_rectangle.x )
+      ( x < sampler->sampler_rectangle[0].x )
       ||
-      ( y < sampler->sampler_rectangle.y )
+      ( y < sampler->sampler_rectangle[0].y )
       ||
-      ( x >= sampler->sampler_rectangle.x + sampler->sampler_rectangle.width )
+      ( x >= sampler->sampler_rectangle[0].x + sampler->sampler_rectangle[0].width )
       ||
-      ( y >= sampler->sampler_rectangle.y + sampler->sampler_rectangle.height ))
+      ( y >= sampler->sampler_rectangle[0].y + sampler->sampler_rectangle[0].height ))
     {
       /*
        * fetch_rectangle will become the value of
@@ -347,19 +333,19 @@ gegl_sampler_get_from_buffer (GeglSampler *const sampler,
       GeglRectangle fetch_rectangle;
 
       fetch_rectangle.x =
-        x - ( maximum_width_and_height - sampler->context_rect.width  ) / 8;
+        x - ( maximum_width_and_height - sampler->context_rect[0].width  ) / 8;
       fetch_rectangle.y =
-        y - ( maximum_width_and_height - sampler->context_rect.height ) / 8;
+        y - ( maximum_width_and_height - sampler->context_rect[0].height ) / 8;
 
       fetch_rectangle.width  = maximum_width_and_height;
       fetch_rectangle.height = maximum_width_and_height;
 
-      if (sampler->sampler_buffer == NULL)
+      if (sampler->sampler_buffer[0] == NULL)
         {
           /*
            * Always request the same amount of pixels:
            */
-          sampler->sampler_buffer =
+          sampler->sampler_buffer[0] =
             g_malloc0 (( maximum_width_and_height * maximum_width_and_height )
                        * bpp);
         }
@@ -368,16 +354,111 @@ gegl_sampler_get_from_buffer (GeglSampler *const sampler,
                        1.0,
                        &fetch_rectangle,
                        sampler->interpolate_format,
-                       sampler->sampler_buffer,
+                       sampler->sampler_buffer[0],
                        GEGL_AUTO_ROWSTRIDE);
 
-      sampler->sampler_rectangle = fetch_rectangle;
+      sampler->sampler_rectangle[0] = fetch_rectangle;
     }
 
-  dx = x - sampler->sampler_rectangle.x;
-  dy = y - sampler->sampler_rectangle.y;
-  buffer_ptr = (guchar *)sampler->sampler_buffer;
-  sof = ( dx + dy * sampler->sampler_rectangle.width ) * bpp;
+  dx = x - sampler->sampler_rectangle[0].x;
+  dy = y - sampler->sampler_rectangle[0].y;
+  buffer_ptr = (guchar *)sampler->sampler_buffer[0];
+  sof = ( dx + dy * sampler->sampler_rectangle[0].width ) * bpp;
+
+  return (gfloat*)(buffer_ptr+sof);
+}
+
+gfloat *
+gegl_sampler_get_from_mipmap (GeglSampler *const sampler,
+                              const gint         x,
+                              const gint         y,
+                              const gint         level)
+{
+  guchar *buffer_ptr;
+  gint    dx;
+  gint    dy;
+  gint    sof;
+
+  const gdouble scale = 1. / ( (gdouble) (1<<level) );
+
+  const gint bpp =
+    babl_format_get_bytes_per_pixel (sampler->interpolate_format);
+
+  /*
+   * maximum_width_and_height is the largest number of pixels which
+   * can be be requested in the horizontal or vertical directions (64
+   * in GEGL).
+   */
+  const gint maximum_width_and_height = 64;
+  g_assert (sampler->context_rect[level].width  <= maximum_width_and_height);
+  g_assert (sampler->context_rect[level].height <= maximum_width_and_height);
+  g_assert (level >= 0 && level < GEGL_SAMPLER_MIPMAP_LEVELS);
+
+  if (( sampler->sampler_buffer[level] == NULL )
+      ||
+      ( x + sampler->context_rect[level].x < sampler->sampler_rectangle[level].x )
+      ||
+      ( y + sampler->context_rect[level].y < sampler->sampler_rectangle[level].y )
+      ||
+      ( x + sampler->context_rect[level].x + sampler->context_rect[level].width
+        > sampler->sampler_rectangle[level].x + sampler->sampler_rectangle[level].width )
+      ||
+      ( y + sampler->context_rect[level].y + sampler->context_rect[level].height
+        > sampler->sampler_rectangle[level].y + sampler->sampler_rectangle[level].height ))
+    {
+      /*
+       * fetch_rectangle will become the value of
+       * sampler->sampler_rectangle[level]:
+       */
+      GeglRectangle fetch_rectangle;
+
+      /*
+       * Override the fetch rectangle needed by the sampler, hoping
+       * that the extra pixels are useful for subsequent requests,
+       * assuming that it is more likely that further access is to the
+       * right or down of our currently requested
+       * position. Consequently, we move the top left corner of the
+       * context_rect by about one fourth of the maximal distance we
+       * can (one fourth of one half = one eight). Given that the
+       * maximum width and height of the fetch_rectangle is 64, so
+       * that half of it is 32, one fourth of the elbow room is at
+       * most 8. If context_rect is large, the corner is not moved
+       * much if at all, as should be.
+       */
+      fetch_rectangle.x =
+        x + sampler->context_rect[level].x
+        - ( maximum_width_and_height - sampler->context_rect[level].width  ) / 8;
+      fetch_rectangle.y =
+        y + sampler->context_rect[level].y
+        - ( maximum_width_and_height - sampler->context_rect[level].height ) / 8;
+
+      fetch_rectangle.width  = maximum_width_and_height;
+      fetch_rectangle.height = maximum_width_and_height;
+
+      if (sampler->sampler_buffer[level] == NULL)
+        {
+          /*
+           * Always request the same amount of pixels:
+           */
+          sampler->sampler_buffer[level] =
+            g_malloc0 (( maximum_width_and_height * maximum_width_and_height )
+                       * bpp);
+        }
+
+      gegl_buffer_get (sampler->buffer,
+                       scale,
+                       &fetch_rectangle,
+                       sampler->interpolate_format,
+                       sampler->sampler_buffer[level],
+                       GEGL_AUTO_ROWSTRIDE);
+
+      sampler->sampler_rectangle[level] = fetch_rectangle;
+    }
+
+  dx = x - sampler->sampler_rectangle[level].x;
+  dy = y - sampler->sampler_rectangle[level].y;
+  buffer_ptr = (guchar *)sampler->sampler_buffer[level];
+  sof = ( dx + dy * sampler->sampler_rectangle[level].width ) * bpp;
 
   return (gfloat*)(buffer_ptr+sof);
 }
@@ -400,10 +481,6 @@ get_property (GObject    *object,
         g_value_set_pointer (value, self->format);
         break;
 
-      case PROP_INVERSE_JACOBIAN:
-        g_value_set_pointer (value, self->inverse_jacobian);
-        break;
-
       default:
         break;
     }
@@ -427,10 +504,6 @@ set_property (GObject      *object,
         self->format = g_value_get_pointer (value);
         break;
 
-      case PROP_INVERSE_JACOBIAN:
-        self->inverse_jacobian = g_value_get_pointer (value);
-        break;
-
       default:
         break;
     }
@@ -450,68 +523,76 @@ set_buffer (GeglSampler *self, GeglBuffer *buffer)
      }
 }
 
-GeglInterpolation
-gegl_buffer_interpolation_from_string (const gchar *string)
+GeglSamplerType
+gegl_sampler_type_from_string (const gchar *string)
 {
   if (g_str_equal (string, "nearest") ||
       g_str_equal (string, "none"))
-    return GEGL_INTERPOLATION_NEAREST;
+    return GEGL_SAMPLER_NEAREST;
 
   if (g_str_equal (string, "linear") ||
       g_str_equal (string, "bilinear"))
-    return GEGL_INTERPOLATION_LINEAR;
+    return GEGL_SAMPLER_LINEAR;
 
   if (g_str_equal (string, "cubic") ||
       g_str_equal (string, "bicubic"))
-    return GEGL_INTERPOLATION_CUBIC;
+    return GEGL_SAMPLER_CUBIC;
 
-  if (g_str_equal (string, "downsharp"))
-    return GEGL_INTERPOLATION_DOWNSHARP;
+  if (g_str_equal (string, "lohalo"))
+    return GEGL_SAMPLER_LOHALO;
 
-  if (g_str_equal (string, "downsize"))
-    return GEGL_INTERPOLATION_DOWNSIZE;
-
-  if (g_str_equal (string, "downsmooth"))
-    return GEGL_INTERPOLATION_DOWNSMOOTH;
-
-  if (g_str_equal (string, "upsharp"))
-    return GEGL_INTERPOLATION_UPSHARP;
-
-  if (g_str_equal (string, "upsize"))
-    return GEGL_INTERPOLATION_UPSIZE;
-
-  if (g_str_equal (string, "upsmooth"))
-    return GEGL_INTERPOLATION_UPSMOOTH;
-
-  return GEGL_INTERPOLATION_NEAREST;
+  return GEGL_SAMPLER_NEAREST;
 }
 
 GType
-gegl_sampler_type_from_interpolation (GeglInterpolation interpolation)
+gegl_sampler_gtype_from_enum (GeglSamplerType sampler_type);
+GType
+gegl_sampler_gtype_from_enum (GeglSamplerType sampler_type)
 {
-  switch (interpolation)
+  switch (sampler_type)
     {
-      case GEGL_INTERPOLATION_NEAREST:
+      case GEGL_SAMPLER_NEAREST:
         return GEGL_TYPE_SAMPLER_NEAREST;
-      case GEGL_INTERPOLATION_LINEAR:
+      case GEGL_SAMPLER_LINEAR:
         return GEGL_TYPE_SAMPLER_LINEAR;
-      case GEGL_INTERPOLATION_CUBIC:
+      case GEGL_SAMPLER_CUBIC:
         return GEGL_TYPE_SAMPLER_CUBIC;
-      case GEGL_INTERPOLATION_LANCZOS:
+      case GEGL_SAMPLER_LANCZOS:
         return GEGL_TYPE_SAMPLER_LANCZOS;
-      case GEGL_INTERPOLATION_DOWNSHARP:
-        return GEGL_TYPE_SAMPLER_DOWNSHARP;
-      case GEGL_INTERPOLATION_DOWNSIZE:
-        return GEGL_TYPE_SAMPLER_DOWNSIZE;
-      case GEGL_INTERPOLATION_DOWNSMOOTH:
-        return GEGL_TYPE_SAMPLER_DOWNSMOOTH;
-      case GEGL_INTERPOLATION_UPSHARP:
-        return GEGL_TYPE_SAMPLER_UPSHARP;
-      case GEGL_INTERPOLATION_UPSIZE:
-        return GEGL_TYPE_SAMPLER_UPSIZE;
-      case GEGL_INTERPOLATION_UPSMOOTH:
-        return GEGL_TYPE_SAMPLER_UPSMOOTH;
+      case GEGL_SAMPLER_LOHALO:
+        return GEGL_TYPE_SAMPLER_LOHALO;
       default:
         return GEGL_TYPE_SAMPLER_LINEAR;
     }
 }
+
+GeglSampler *
+gegl_buffer_sampler_new (GeglBuffer       *buffer,
+                         Babl             *format,
+                         GeglSamplerType   sampler_type)
+{
+  GeglSampler          *sampler;
+  GType                 desired_type;
+  if (format == NULL)
+    format = babl_format ("RaGaBaA float");
+  desired_type = gegl_sampler_gtype_from_enum (sampler_type);
+  if (sampler_type == GEGL_SAMPLER_LANCZOS)
+      sampler = g_object_new (desired_type,
+                              "format", format,
+                              "buffer", buffer,
+                              "lanczos_width",  4,
+                              NULL);
+  else
+      sampler = g_object_new (desired_type,
+                              "buffer", buffer,
+                              "format", format,
+                              NULL);
+  gegl_sampler_prepare (sampler);
+  return sampler;
+}
+
+const GeglRectangle*
+gegl_sampler_get_context_rect (GeglSampler *sampler)
+{
+  return &(sampler->context_rect[0]);
+}
