@@ -31,23 +31,34 @@ G_BEGIN_DECLS
 #define GEGL_IS_SAMPLER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_SAMPLER))
 #define GEGL_IS_SAMPLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GEGL_TYPE_SAMPLER))
 #define GEGL_SAMPLER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_SAMPLER, GeglSamplerClass))
+#define GEGL_SAMPLER_MIPMAP_LEVELS   3
 
 typedef struct _GeglSamplerClass GeglSamplerClass;
-typedef struct _GeglSampler    GeglSampler;
 
 struct _GeglSampler
 {
-  GObject        parent_instance;
+  GObject       parent_instance;
+  void (* get) (GeglSampler *self,
+                gdouble      x,
+                gdouble      y,
+                GeglMatrix2 *scale,
+                void        *output);
+  /* we cache the getter in the instance, (being able to return the
+     function pointer itself and cache it outside the calling loop
+     would be even quicker.
+   */
 
   /*< private >*/
   GeglBuffer    *buffer;
   Babl          *format;
   Babl          *interpolate_format;
-  GeglRectangle  context_rect;
-  void          *sampler_buffer;
-  GeglRectangle  sampler_rectangle;
+  Babl          *fish;
+  GeglRectangle  context_rect[GEGL_SAMPLER_MIPMAP_LEVELS];
+  gpointer       sampler_buffer[GEGL_SAMPLER_MIPMAP_LEVELS];
+  GeglRectangle  sampler_rectangle[GEGL_SAMPLER_MIPMAP_LEVELS];
   gdouble        x; /* mirrors the currently requested */
   gdouble        y; /* coordinates in the instance     */
+
 };
 
 struct _GeglSamplerClass
@@ -58,6 +69,7 @@ struct _GeglSamplerClass
   void (* get)       (GeglSampler *self,
                       gdouble      x,
                       gdouble      y,
+                      GeglMatrix2 *scale,
                       void        *output);
  void  (*set_buffer) (GeglSampler  *self,
                       GeglBuffer   *buffer);
@@ -73,16 +85,20 @@ void  gegl_sampler_set_buffer  (GeglSampler *self,
 void  gegl_sampler_get         (GeglSampler *self,
                                 gdouble      x,
                                 gdouble      y,
+                                GeglMatrix2 *scale,
                                 void        *output);
+
 gfloat * gegl_sampler_get_from_buffer (GeglSampler *sampler,
                                        gint         x,
                                        gint         y);
-
+gfloat * gegl_sampler_get_from_mipmap (GeglSampler *sampler,
+                                       gint         x,
+                                       gint         y,
+                                       gint         level);
 gfloat *
 gegl_sampler_get_ptr (GeglSampler         *sampler,
                       gint                 x,
                       gint                 y);
-GType gegl_sampler_type_from_interpolation (GeglInterpolation interpolation);
 
 G_END_DECLS
 
