@@ -115,6 +115,9 @@ gegl_tile_dup (GeglTile *src)
   tile->data       = src->data;
   tile->size       = src->size;
 
+  tile->destroy_notify      = src->destroy_notify;
+  tile->destroy_notify_data = src->destroy_notify_data;
+
   tile->next_shared              = src->next_shared;
   src->next_shared               = tile;
   tile->prev_shared              = src;
@@ -160,6 +163,8 @@ gegl_tile_unclone (GeglTile *tile)
        * create a local copy
        */
       tile->data                     = gegl_memdup (tile->data, tile->size);
+      tile->destroy_notify           = default_free;
+      tile->destroy_notify_data      = NULL;
       tile->prev_shared->next_shared = tile->next_shared;
       tile->next_shared->prev_shared = tile->prev_shared;
       tile->prev_shared              = tile;
@@ -208,11 +213,11 @@ _gegl_tile_void_pyramid (GeglTileSource *source,
 static void
 gegl_tile_void_pyramid (GeglTile *tile)
 {
-  if (tile->tile_storage && 
+  if (tile->tile_storage &&
       tile->tile_storage->seen_zoom &&
       tile->z == 0) /* we only accepting voiding the base level */
     {
-      _gegl_tile_void_pyramid (GEGL_TILE_SOURCE (tile->tile_storage), 
+      _gegl_tile_void_pyramid (GEGL_TILE_SOURCE (tile->tile_storage),
                                tile->x/2,
                                tile->y/2,
                                tile->z+1);
@@ -226,6 +231,12 @@ gegl_tile_unlock (GeglTile *tile)
 #if 0
   total_unlocks++;
 #endif
+
+  if (tile->unlock_notify != NULL)
+    {
+      tile->unlock_notify (tile, tile->unlock_notify_data);
+    }
+
   if (tile->lock == 0)
     {
       g_warning ("unlocked a tile with lock count == 0");
@@ -276,13 +287,33 @@ gboolean gegl_tile_store (GeglTile *tile)
                                     tile);
 }
 
-/* for internal use, a macro poking directly at the data will be faster 
+/* for internal use, a macro poking directly at the data will be faste
  */
 guchar *gegl_tile_get_data (GeglTile *tile)
 {
   return tile->data;
 }
 
+void gegl_tile_set_data (GeglTile *tile,
+                         gpointer  pixel_data,
+                         gint      pixel_data_size)
+{
+  tile->data = pixel_data;
+  tile->size = pixel_data_size;
+}
+
+void gegl_tile_set_data_full (GeglTile         *tile,
+                              gpointer          pixel_data,
+                              gint              pixel_data_size,
+                              GeglDestroyNotify destroy_notify,
+                              gpointer          destroy_notify_data)
+{
+  tile->data                = pixel_data;
+  tile->size                = pixel_data_size;
+  tile->destroy_notify      = destroy_notify;
+  tile->destroy_notify_data = destroy_notify_data;
+}
+
 
 void         gegl_tile_set_rev        (GeglTile *tile,
                                        guint     rev)
@@ -294,3 +325,11 @@ guint        gegl_tile_get_rev        (GeglTile *tile)
 {
   return tile->rev;
 }
+
+void gegl_tile_set_unlock_notify (GeglTile         *tile,
+                                  GeglTileCallback  unlock_notify,
+                                  gpointer          unlock_notify_data)
+{
+  tile->unlock_notify      = unlock_notify;
+  tile->unlock_notify_data = unlock_notify_data;
+}
