@@ -28,6 +28,7 @@
 
 #include <gegl-plugin.h>
 
+G_BEGIN_DECLS
 
 GType gegl_chant_get_type (void);
 typedef struct _GeglChantO  GeglChantO;
@@ -60,7 +61,6 @@ type_name##_register_type (GTypeModule *type_module)                    \
   {                                                                     \
     gchar  tempname[256];                                               \
     gchar *p;                                                           \
-    GType  g_define_type_id;                                            \
     const GTypeInfo g_define_type_info =                                \
     {                                                                   \
       sizeof (TypeName##Class),                                         \
@@ -84,7 +84,6 @@ type_name##_register_type (GTypeModule *type_module)                    \
                                                        tempname,        \
                                                        &g_define_type_info, \
                                                        (GTypeFlags) flags); \
-    g_define_type_id = type_name##_type_id;                             \
     { CODE ; }                                                          \
   }
 
@@ -343,18 +342,19 @@ struct _GeglChantO
                         * in finalize). Also serves as a filler making sure that we
                         * do not create an empty struct if there are no chanted properties.
                         */
-#define gegl_chant_int(name, nick, min, max, def, blurb)     gint        name;
-#define gegl_chant_double(name, nick, min, max, def, blurb)  gdouble     name;
-#define gegl_chant_boolean(name, nick, def, blurb)           gboolean    name;
-#define gegl_chant_string(name, nick, def, blurb)            gchar      *name;
-#define gegl_chant_file_path(name, nick, def, blurb)         gchar      *name;
-#define gegl_chant_multiline(name, nick, def, blurb)         gchar      *name;
-#define gegl_chant_object(name,nick,  blurb)                 GObject    *name;
-#define gegl_chant_pointer(name, nick, blurb)                gpointer    name;
-#define gegl_chant_color(name, nick, def, blurb)             GeglColor  *name;
-#define gegl_chant_curve(name, nick, blurb)                  GeglCurve  *name;
-#define gegl_chant_path(name, nick, blurb)                   GeglPath   *name;\
-                                                   guint path_changed_handler;
+#define gegl_chant_int(name, nick, min, max, def, blurb)     gint               name;
+#define gegl_chant_double(name, nick, min, max, def, blurb)  gdouble            name;
+#define gegl_chant_boolean(name, nick, def, blurb)           gboolean           name;
+#define gegl_chant_string(name, nick, def, blurb)            gchar             *name;
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)  enum               name;
+#define gegl_chant_file_path(name, nick, def, blurb)         gchar             *name;
+#define gegl_chant_multiline(name, nick, def, blurb)         gchar             *name;
+#define gegl_chant_object(name,nick,  blurb)                 GObject           *name;
+#define gegl_chant_pointer(name, nick, blurb)                gpointer           name;
+#define gegl_chant_color(name, nick, def, blurb)             GeglColor         *name;
+#define gegl_chant_curve(name, nick, blurb)                  GeglCurve         *name;
+#define gegl_chant_path(name, nick, blurb)                   GeglPath          *name;\
+                                                          guint path_changed_handler;
 
 #include GEGL_CHANT_C_FILE
 
@@ -362,6 +362,7 @@ struct _GeglChantO
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -380,6 +381,7 @@ enum
 #define gegl_chant_double(name, nick, min, max, def, blurb)  PROP_##name,
 #define gegl_chant_boolean(name, nick, def, blurb)           PROP_##name,
 #define gegl_chant_string(name, nick, def, blurb)            PROP_##name,
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)  PROP_##name,
 #define gegl_chant_file_path(name, nick, def, blurb)         PROP_##name,
 #define gegl_chant_multiline(name, nick, def, blurb)         PROP_##name,
 #define gegl_chant_object(name, nick, blurb)                 PROP_##name,
@@ -394,6 +396,7 @@ enum
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -432,6 +435,10 @@ get_property (GObject      *gobject,
     case PROP_##name:                                         \
       g_value_set_string (value, properties->name);           \
       break;
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)   \
+    case PROP_##name:                                         \
+      g_value_set_enum (value, properties->name);             \
+      break;
 #define gegl_chant_file_path(name, nick, def, blurb)          \
     case PROP_##name:                                         \
       g_value_set_string (value, properties->name);           \
@@ -468,6 +475,7 @@ get_property (GObject      *gobject,
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -479,6 +487,7 @@ get_property (GObject      *gobject,
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
       break;
   }
+  properties ++; /* evil hack to silence gcc */
 }
 
 static void
@@ -511,6 +520,10 @@ set_property (GObject      *gobject,
         g_free (properties->name);                                    \
       properties->name = g_strdup (g_value_get_string (value));       \
       break;
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)           \
+    case PROP_##name:                                                 \
+      properties->name = g_value_get_enum (value);                    \
+      break;
 #define gegl_chant_file_path(name, nick, def, blurb)                  \
     case PROP_##name:                                                 \
       if (properties->name)                                           \
@@ -568,6 +581,7 @@ set_property (GObject      *gobject,
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -580,6 +594,8 @@ set_property (GObject      *gobject,
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
       break;
   }
+
+  properties ++; /* evil hack to silence gcc */
 }
 
 static void gegl_chant_destroy_notify (gpointer data)
@@ -595,6 +611,7 @@ static void gegl_chant_destroy_notify (gpointer data)
       g_free (properties->name);                    \
       properties->name = NULL;                      \
     }
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)
 #define gegl_chant_file_path(name, nick, def, blurb) \
   if (properties->name)                             \
     {                                               \
@@ -639,6 +656,7 @@ static void gegl_chant_destroy_notify (gpointer data)
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -664,14 +682,15 @@ gegl_chant_constructor (GType                  type,
   /* create dummy colors and vectors */
   properties = GEGL_CHANT_PROPERTIES (obj);
 
-#define gegl_chant_int(name, nick, min, max, def, blurb)              
-#define gegl_chant_double(name, nick, min, max, def, blurb)           
-#define gegl_chant_boolean(name, nick, def, blurb)                    
-#define gegl_chant_string(name, nick, def, blurb)                     
-#define gegl_chant_file_path(name, nick, def, blurb)                  
-#define gegl_chant_multiline(name, nick, def, blurb)                  
-#define gegl_chant_object(name, nick, blurb)                          
-#define gegl_chant_pointer(name, nick, blurb)                         
+#define gegl_chant_int(name, nick, min, max, def, blurb)
+#define gegl_chant_double(name, nick, min, max, def, blurb)
+#define gegl_chant_boolean(name, nick, def, blurb)
+#define gegl_chant_string(name, nick, def, blurb)
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)
+#define gegl_chant_file_path(name, nick, def, blurb)
+#define gegl_chant_multiline(name, nick, def, blurb)
+#define gegl_chant_object(name, nick, blurb)
+#define gegl_chant_pointer(name, nick, blurb)
 #define gegl_chant_color(name, nick, def, blurb)              \
     if (properties->name == NULL) \
     {properties->name = gegl_color_new(def?def:"black");}
@@ -684,6 +703,7 @@ gegl_chant_constructor (GType                  type,
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -692,8 +712,8 @@ gegl_chant_constructor (GType                  type,
 #undef gegl_chant_curve
 #undef gegl_chant_path
 
-
   g_object_set_data_full (obj, "chant-data", obj, gegl_chant_destroy_notify);
+  properties ++; /* evil hack to silence gcc */
   return obj;
 }
 
@@ -738,6 +758,15 @@ gegl_chant_class_intern_init (gpointer klass)
                                                         G_PARAM_READWRITE |  \
                                                         G_PARAM_CONSTRUCT |  \
                                                         GEGL_PARAM_PAD_INPUT)));
+#define gegl_chant_enum(name, nick, enum, type, def, blurb)                  \
+  g_object_class_install_property (object_class, PROP_##name,                \
+                                   g_param_spec_enum (#name, nick, blurb,    \
+                                                      type,                  \
+                                                      def,                   \
+                                                      (GParamFlags) (        \
+                                                      G_PARAM_READWRITE |    \
+                                                      G_PARAM_CONSTRUCT |    \
+                                                      GEGL_PARAM_PAD_INPUT)));
 #define gegl_chant_file_path(name, nick, def, blurb)                              \
   g_object_class_install_property (object_class, PROP_##name,                \
                                    gegl_param_spec_file_path (#name, nick, blurb, \
@@ -776,7 +805,7 @@ gegl_chant_class_intern_init (gpointer klass)
                                                           def,               \
                                                           (GParamFlags) (    \
                                                           G_PARAM_READWRITE |\
-														  G_PARAM_CONSTRUCT | \
+                                                          G_PARAM_CONSTRUCT | \
                                                           GEGL_PARAM_PAD_INPUT)));
 #define gegl_chant_path(name, nick, blurb)                                 \
   g_object_class_install_property (object_class, PROP_##name,                \
@@ -795,13 +824,13 @@ gegl_chant_class_intern_init (gpointer klass)
                                                           G_PARAM_CONSTRUCT |\
                                                           GEGL_PARAM_PAD_INPUT)));
 
-
 #include GEGL_CHANT_C_FILE
 
 #undef gegl_chant_int
 #undef gegl_chant_double
 #undef gegl_chant_boolean
 #undef gegl_chant_string
+#undef gegl_chant_enum
 #undef gegl_chant_file_path
 #undef gegl_chant_multiline
 #undef gegl_chant_object
@@ -818,4 +847,5 @@ gegl_chant_init_properties (GeglChant *self)
   self->properties = g_slice_new0 (GeglChantO);
 }
 
+G_END_DECLS
 /****************************************************************************/
