@@ -17,9 +17,6 @@
  */
 #include "config.h"
 
-/* For clang, remove when getting rid of using PATH_MAX */
-#define _XOPEN_SOURCE 500
-
 #include <glib.h>
 #include <glib-object.h>
 #include <stdlib.h>
@@ -40,9 +37,6 @@
 #include "gegl-xml.h"
 
 #ifdef G_OS_WIN32
-#ifndef PATH_MAX
-#define PATH_MAX _MAX_PATH
-#endif
 #define realpath(a, b)    _fullpath (b, a, _MAX_PATH)
 #endif
 
@@ -129,7 +123,7 @@ set_clone_prop_as_well:
       else if (g_type_is_a (G_PARAM_SPEC_TYPE (paramspec),
                             GEGL_TYPE_PARAM_FILE_PATH))
         {
-          gchar buf[PATH_MAX];
+          gchar *buf;
 
           if (g_path_is_absolute (param_value))
             {
@@ -140,15 +134,15 @@ set_clone_prop_as_well:
               gchar * absolute_path;
               if (pd->path_root)
                 {
-                  g_snprintf (buf, sizeof (buf),
-                              "%s/%s", pd->path_root, param_value);
+                  buf = g_strdup_printf ("%s/%s", pd->path_root, param_value);
                 }
               else
                 {
-                  g_snprintf (buf, sizeof (buf), "./%s", param_value);
+                  buf = g_strdup_printf ("./%s", param_value);
                 }
 
               absolute_path = realpath (buf, NULL);
+              g_free (buf);
               if (absolute_path)
                 {
                   gegl_node_set (new, param_name, absolute_path, NULL);
@@ -183,8 +177,7 @@ set_clone_prop_as_well:
               !strcmp (param_value, "y") ||
               !strcmp (param_value, "Y") ||
               !strcmp (param_value, "1") ||
-              !strcmp (param_value, "on")
-          )
+              !strcmp (param_value, "on"))
             {
               gegl_node_set (new, param_name, TRUE, NULL);
             }
@@ -193,6 +186,12 @@ set_clone_prop_as_well:
               gegl_node_set (new, param_name, FALSE, NULL);
             }
         }
+      else if (g_type_is_a (paramspec->value_type, G_TYPE_ENUM))
+        {
+          GEnumClass *eclass = g_type_class_peek (paramspec->value_type);
+          GEnumValue *evalue = g_enum_get_value_by_nick (eclass, param_value);
+          gegl_node_set (new, param_name, evalue->value, NULL);
+        }
       else if (paramspec->value_type == GEGL_TYPE_COLOR)
         {
           GeglColor *color = g_object_new (GEGL_TYPE_COLOR,
@@ -205,20 +204,20 @@ set_clone_prop_as_well:
         }
       else if (paramspec->value_type == GEGL_TYPE_CURVE)
         {
-	  if (pd->curve)
-	    {
-	      gegl_node_set (new, param_name, pd->curve, NULL);
+          if (pd->curve)
+            {
+              gegl_node_set (new, param_name, pd->curve, NULL);
 
-	      g_object_unref (pd->curve);
-	      pd->curve = NULL;
-	    }
-	}
+              g_object_unref (pd->curve);
+              pd->curve = NULL;
+            }
+        }
       else if (paramspec->value_type == GEGL_TYPE_PATH)
         {
           GeglPath *path = gegl_path_new ();
           gegl_path_parse_string (path, param_value);
-	  gegl_node_set (new, param_name, path, NULL);
-	}
+          gegl_node_set (new, param_name, path, NULL);
+        }
       else
         {
           g_warning ("operation desired unknown parapspec type for %s",
@@ -302,28 +301,28 @@ static void start_element (GMarkupParseContext *context,
     {
       const gchar *ymin, *ymax;
       if (pd->curve != NULL)
-	g_warning ("we haven't cleared previous curve");
+        g_warning ("we haven't cleared previous curve");
 
       collect_attribute (ymin, "curve");
       collect_attribute (ymax, "curve");
 
       pd->curve = gegl_curve_new (g_ascii_strtod (ymin, NULL),
-				  g_ascii_strtod (ymax, NULL));
+                                  g_ascii_strtod (ymax, NULL));
     }
   else if (!strcmp (element_name, "curve-point"))
     {
       if (!pd->curve)
-	g_warning ("curve not instantiated");
+        g_warning ("curve not instantiated");
       else
         {
           const gchar *x, *y;
           collect_attribute (x, "curve-point");
           collect_attribute (y, "curve-point");
 
-	  gegl_curve_add_point (pd->curve,
-				g_ascii_strtod (x, NULL),
-				g_ascii_strtod (y, NULL));
-	}
+          gegl_curve_add_point (pd->curve,
+                                g_ascii_strtod (x, NULL),
+                                g_ascii_strtod (y, NULL));
+        }
     }
   else if (!strcmp (element_name, "link") ||
            !strcmp (element_name, "links") ||
@@ -469,7 +468,7 @@ static void end_element (GMarkupParseContext *context,
            !strcmp (element_name, "destination") ||
            !strcmp (element_name, "stack") ||
            !strcmp (element_name, "params") ||
-	   !strcmp (element_name, "curve-point"))
+           !strcmp (element_name, "curve-point"))
     {
       /* ignore */
     }
@@ -516,7 +515,7 @@ static void each_ref (gpointer value,
 
   gegl_node_get (dest_node, "ref", &ref, NULL);
   source_node = g_hash_table_lookup (pd->ids, ref);
-
+  g_free (ref);
   gegl_node_connect_from (dest_node, "input", source_node, "output");
 }
 
@@ -571,13 +570,14 @@ gegl_node_new_from_file (const gchar   *path)
   GeglNode *node = NULL;
   GError   *err  = NULL;
   gchar    *script;
-  gchar     path_root[PATH_MAX];
+  gchar    *path_root;
   gchar    *dirname;
 
   g_assert (path);
 
   dirname = g_path_get_dirname (path);
-  if (!realpath (dirname, path_root))
+  path_root = realpath (dirname, NULL);
+  if (!path_root)
     {
       goto cleanup;
     }
@@ -593,6 +593,7 @@ gegl_node_new_from_file (const gchar   *path)
   node = gegl_node_new_from_xml (script, path_root);
 
 cleanup:
+  g_free (path_root);
   g_free (dirname);
   return node;
 }
@@ -841,6 +842,17 @@ serialize_properties (SerializeState *ss,
               xml_param (ss, indent + 2, properties[i]->name, value);
               g_free (value);
             }
+          else if (g_type_is_a (properties[i]->value_type, G_TYPE_ENUM))
+            {
+              GEnumClass *eclass = g_type_class_peek (properties[i]->value_type);
+              GEnumValue *evalue;
+              gint value;
+
+              gegl_node_get (node, properties[i]->name, &value, NULL);
+              evalue = g_enum_get_value (eclass, value);
+
+              xml_param (ss, indent + 2, properties[i]->name, evalue->value_nick);
+            }
           else if (properties[i]->value_type == GEGL_TYPE_COLOR)
             {
               GeglColor *color;
@@ -851,28 +863,28 @@ serialize_properties (SerializeState *ss,
               xml_param (ss, indent + 2, properties[i]->name, value);
               g_free (value);
             }
-	  else if (properties[i]->value_type == GEGL_TYPE_CURVE)
-	    {
-	      GeglCurve *curve;
-	      gegl_node_get (node, properties[i]->name, &curve, NULL);
-	      xml_param_start (ss, indent + 2, properties[i]->name);
-	      g_string_append (ss->buf, "\n");
-	      xml_curve (ss, indent + 4, curve);
-	      indent += 2; ind; indent -= 2; xml_param_end (ss);
-	      g_object_unref (curve);
-	    }
-	  else if (properties[i]->value_type == GEGL_TYPE_PATH)
-	    {
-          gchar *svg_path;
-          GeglPath *path;
-	      gegl_node_get (node, properties[i]->name, &path, NULL);
-	      xml_param_start (ss, indent + 2, properties[i]->name);
-          svg_path = gegl_path_to_string (path);	      
-	      g_string_append (ss->buf, svg_path);
-	      xml_param_end (ss);
+          else if (properties[i]->value_type == GEGL_TYPE_CURVE)
+            {
+              GeglCurve *curve;
+              gegl_node_get (node, properties[i]->name, &curve, NULL);
+              xml_param_start (ss, indent + 2, properties[i]->name);
+              g_string_append (ss->buf, "\n");
+              xml_curve (ss, indent + 4, curve);
+              indent += 2; ind; indent -= 2; xml_param_end (ss);
+              g_object_unref (curve);
+            }
+          else if (properties[i]->value_type == GEGL_TYPE_PATH)
+            {
+              gchar *svg_path;
+              GeglPath *path;
+              gegl_node_get (node, properties[i]->name, &path, NULL);
+              xml_param_start (ss, indent + 2, properties[i]->name);
+              svg_path = gegl_path_to_string (path);
+              g_string_append (ss->buf, svg_path);
+              xml_param_end (ss);
 
-          g_object_unref (path);
-	    }
+              g_object_unref (path);
+            }
           else
             {
               g_warning ("%s: serialization of %s properties not implemented",
@@ -987,53 +999,53 @@ add_stack (SerializeState *ss,
                 }
             }
 
-          if (!strcmp (class, "layer"))
+          if (class)
             {
-              serialize_layer (ss, indent, iter);
-            }
-          else
-            {
-              if (aux &&
-                  gegl_pad_get_connected_to (aux))
+              if (!strcmp (class, "layer"))
                 {
-                  GeglPad  *source_pad;
-                  GeglNode *source_node;
-                  source_pad  = gegl_pad_get_connected_to (aux);
-                  source_node = gegl_pad_get_node (source_pad);
-                  {
-                    GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
-                                                         "graph");
-                    if (graph)
-                      source_node = graph;
-                  }
-                  ind; g_string_append (ss->buf, "<node");
-
-                  {
-                    gchar *class;
-                    gchar *name;
-                    gegl_node_get (iter, "operation", &class,
-                                   "name", &name,
-                                   NULL);
-                    if (name[0])
-                      {
-                        xml_attr (ss->buf, "name", name);
-                      }
-                    xml_attr (ss->buf, "operation", class);
-                    if (id != NULL)
-                      xml_attr (ss->buf, "id", id);
-                    g_free (name);
-                    g_free (class);
-                  }
-
-                  g_string_append (ss->buf, ">\n");
-                  serialize_properties (ss, indent + 4, iter);
-                  add_stack (ss, indent + 4, source_node);
-
-                  ind; g_string_append (ss->buf, "</node>\n");
+                  serialize_layer (ss, indent, iter);
                 }
               else
                 {
-                  if (class)
+                  if (aux &&
+                      gegl_pad_get_connected_to (aux))
+                    {
+                      GeglPad  *source_pad;
+                      GeglNode *source_node;
+                      source_pad  = gegl_pad_get_connected_to (aux);
+                      source_node = gegl_pad_get_node (source_pad);
+                      {
+                        GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
+                                                             "graph");
+                        if (graph)
+                          source_node = graph;
+                      }
+                      ind; g_string_append (ss->buf, "<node");
+
+                      {
+                        gchar *class;
+                        gchar *name;
+                        gegl_node_get (iter, "operation", &class,
+                                       "name", &name,
+                                       NULL);
+                        if (name[0])
+                          {
+                            xml_attr (ss->buf, "name", name);
+                          }
+                        xml_attr (ss->buf, "operation", class);
+                        if (id != NULL)
+                          xml_attr (ss->buf, "id", id);
+                        g_free (name);
+                        g_free (class);
+                      }
+
+                      g_string_append (ss->buf, ">\n");
+                      serialize_properties (ss, indent + 4, iter);
+                      add_stack (ss, indent + 4, source_node);
+
+                      ind; g_string_append (ss->buf, "</node>\n");
+                    }
+                  else
                     {
                       if (strcmp (class, "gegl:nop") &&
                           strcmp (class, "gegl:clone"))
