@@ -234,7 +234,7 @@ gegl_node_dispose (GObject *gobject)
 
   {
     gint i;
-    for (i=0; i<4; i++)
+    for (i=0; i<GEGL_MAX_THREADS; i++)
       if (self->priv->eval_mgr[i])
         {
           g_object_unref (self->priv->eval_mgr[i]);
@@ -924,11 +924,10 @@ gegl_node_blit (GeglNode            *self,
   g_return_if_fail (GEGL_IS_NODE (self));
   g_return_if_fail (roi != NULL);
 
-#if 1
   threads = gegl_config ()->threads;
   if (threads > GEGL_MAX_THREADS)
     threads = 1;
-  
+
   if (pool == NULL)
     {
       pool = g_thread_pool_new (spawnrender, NULL, threads, TRUE, NULL);
@@ -936,16 +935,8 @@ gegl_node_blit (GeglNode            *self,
       cond = g_cond_new ();
     }
 
-#if 0
-  if (flags == GEGL_BLIT_DEFAULT)
-    flags = GEGL_BLIT_CACHE;
-#endif
-
-  flags = GEGL_BLIT_DEFAULT; /* force all rendering through this path,
-                              * to have less code to worry about making
-                              * multi thread safe
-                              */
   if (flags == GEGL_BLIT_DEFAULT)
+#if 1  /* multi threaded version */
     {
       ThreadData data[GEGL_MAX_THREADS];
       gint i;
@@ -1022,8 +1013,9 @@ gegl_node_blit (GeglNode            *self,
           g_mutex_unlock (mutex);
         }
     }
-#else  /* thread free version, can be removed */
-    if (flags == GEGL_BLIT_DEFAULT)
+#else /* thread free version, could be removed, left behind in case it 
+         is needed for debugging
+       */
     {
       GeglBuffer *buffer;
 
@@ -1047,19 +1039,19 @@ gegl_node_blit (GeglNode            *self,
         g_object_unref (buffer);
     }
 #endif
-  else  /* these code paths currently not used */
-    if ((flags & GEGL_BLIT_CACHE) ||
-           (flags & GEGL_BLIT_DIRTY))
+  else
+    if ((flags & GEGL_BLIT_CACHE))
     {
       GeglCache *cache = gegl_node_get_cache (self);
       if (!(flags & GEGL_BLIT_DIRTY))
-        { 
+        {
           if (!self->priv->processor)
            self->priv->processor = gegl_node_new_processor (self, roi);
+
           gegl_processor_set_rectangle (self->priv->processor, roi);
-          while (gegl_processor_work (self->priv->processor, NULL)) ;
+          while (gegl_processor_work (self->priv->processor, NULL));
         }
-      if (destination_buf)
+      if (destination_buf && cache)
         {
           gegl_buffer_get (GEGL_BUFFER (cache), scale, roi,
                            format, destination_buf, rowstride);
@@ -1278,7 +1270,7 @@ gegl_node_set_operation_object (GeglNode      *self,
     gegl_node_disconnect_sinks (self);
 
     /* Delete all the pads from the previous operation */
-    while ((old_pads = gegl_node_get_pads (self)) != NULL) 
+    while ((old_pads = gegl_node_get_pads (self)) != NULL)
       {
         gegl_node_remove_pad (self, old_pads->data);
       }
@@ -2006,7 +1998,8 @@ gegl_node_get_cache (GeglNode *node)
        * only "output" pads
        */
       pad = gegl_node_get_pad (node, "output");
-      g_assert (pad);
+      if (!pad)
+        return NULL;
       format = gegl_pad_get_format (pad);
       if (!format)
         {
@@ -2065,6 +2058,7 @@ gegl_node_add_child (GeglNode *self,
 {
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
   g_return_val_if_fail (GEGL_IS_NODE (child), NULL);
+  g_return_val_if_fail (child->priv->parent == NULL, NULL);
 
   self->priv->children = g_slist_prepend (self->priv->children,
                                           g_object_ref (child));
