@@ -21,22 +21,10 @@
 #define __GEGL_MODULE_H__
 
 #include <gmodule.h>
-
-#include "geglmoduletypes.h"
-#include "geglmoduledb.h"
+#include "gegl-plugin.h"
 
 G_BEGIN_DECLS
 
-
-/*  increment the ABI version each time one of the following changes:
- *
- *  - the libgeglmodule implementation (if the change affects modules).
- *  - one of the classes implemented by modules (currently GeglColorDisplay,
- *    GeglColorSelector and GeglController).
- */
-#define GEGL_MODULE_ABI_VERSION 0x0004
-
-
 typedef enum
 {
   GEGL_MODULE_STATE_ERROR,       /* missing gegl_module_register function
@@ -52,18 +40,6 @@ typedef enum
                                   */
 } GeglModuleState;
 
-
-struct _GeglModuleInfo
-{
-  guint32  abi_version;
-  gchar   *purpose;
-  gchar   *author;
-  gchar   *version;
-  gchar   *copyright;
-  gchar   *date;
-};
-
-
 typedef const GeglModuleInfo * (* GeglModuleQueryFunc)    (GTypeModule *module);
 typedef gboolean               (* GeglModuleRegisterFunc) (GTypeModule *module);
 
@@ -133,12 +109,7 @@ const gchar * gegl_module_state_name       (GeglModuleState  state);
 
 /*  GeglModuleInfo functions  */
 
-GeglModuleInfo * gegl_module_info_new  (guint32               abi_version,
-                                        const gchar          *purpose,
-                                        const gchar          *author,
-                                        const gchar          *version,
-                                        const gchar          *copyright,
-                                        const gchar          *date);
+GeglModuleInfo * gegl_module_info_new  (guint32               abi_version);
 GeglModuleInfo * gegl_module_info_copy (const GeglModuleInfo *info);
 void             gegl_module_info_free (GeglModuleInfo       *info);
 
