@@ -16,6 +16,11 @@
  * Copyright 2006 Øyvind Kolås
  */
 
+/* GeglOperationSink
+ * An operation that consumes a GeglBuffer, used for filewriters, display 
+ * (for the sdl display node)
+ */
+
 #ifndef __GEGL_OPERATION_SINK_H__
 #define __GEGL_OPERATION_SINK_H__
 
@@ -41,11 +46,14 @@ struct _GeglOperationSinkClass
 {
   GeglOperationClass parent_class;
 
+  /* Wether or not the sink operation needs full input data in one go
+   * in order to be able to do its processing
+   */
   gboolean           needs_full;
 
   gboolean (* process) (GeglOperation       *self,
                         GeglBuffer          *input,
-                        const GeglRectangle *result);
+                        const GeglRectangle *roi);
 };
 
 GType    gegl_operation_sink_get_type   (void) G_GNUC_CONST;
