@@ -16,15 +16,17 @@
  * Copyright 2006 Øyvind Kolås
  */
 
-#define GEGL_INTERNAL
 #include "config.h"
 
+#include <string.h>
+
 #include <glib-object.h>
-#include "gegl-types.h"
+
+#include "gegl.h"
+#include "gegl-types-internal.h"
 #include "gegl-operation-source.h"
 #include "graph/gegl-node.h"
 #include "graph/gegl-pad.h"
-#include <string.h>
 
 enum
 {
@@ -41,7 +43,7 @@ static void     set_property (GObject      *gobject,
                               guint         prop_id,
                               const GValue *value,
                               GParamSpec   *pspec);
-static gboolean gegl_operation_source_process 
+static gboolean gegl_operation_source_process
                              (GeglOperation *operation,
                               GeglOperationContext *context,
                               const gchar   *output_prop,
@@ -137,6 +139,9 @@ gegl_operation_source_process (GeglOperation       *operation,
   output = gegl_operation_context_get_target (context, "output");
   success = klass->process (operation, output, result);
 
+  if (output == GEGL_BUFFER (operation->node->cache))
+    gegl_cache_computed (operation->node->cache, result);
+
   return success;
 }
 
@@ -145,8 +150,10 @@ get_bounding_box (GeglOperation *self)
 {
   GeglRectangle result = { 0, 0, 0, 0 };
 
-  g_warning ("Gegl Source '%s' has no proper have_rect function",
-             G_OBJECT_CLASS_NAME (G_OBJECT_GET_CLASS (self)));
+  g_warning ("Gegl Source '%s' does not override %s()",
+             G_OBJECT_CLASS_NAME (G_OBJECT_GET_CLASS (self)),
+             G_STRFUNC);
+
   return result;
 }
 
@@ -162,7 +169,5 @@ static GeglRectangle
 get_cached_region (GeglOperation       *operation,
                    const GeglRectangle *roi)
 {
-  GeglRectangle result = *roi;
-  result = operation->node->have_rect;
-  return result;
+  return *roi;
 }
