@@ -276,7 +276,13 @@ gegl_operation_source_get_bounding_box (GeglOperation  *operation,
   GeglNode *node = gegl_operation_get_source_node (operation, input_pad_name);
 
   if (node)
-    return &node->have_rect;
+    {
+      GeglRectangle *ret;
+      g_mutex_lock (node->mutex);
+      ret = &node->have_rect;
+      g_mutex_unlock (node->mutex);
+      return ret;
+    }
 
   return NULL;
 }
