@@ -56,7 +56,6 @@ gegl_eval_visitor_init (GeglEvalVisitor *self)
 {
 }
 
-extern long babl_total_usecs;
 
 /* this is the visitor that does the real computations for GEGL */
 static void
@@ -83,17 +82,32 @@ gegl_eval_visitor_visit_pad (GeglVisitor *self,
       else
         {
           glong time      = gegl_ticks ();
-          glong babl_time = babl_total_usecs;
 
           /* Make the operation do it's actual processing */
-          GEGL_NOTE (GEGL_DEBUG_PROCESS, "Processing pad '%s' on \"%s\"", gegl_pad_get_name (pad), gegl_node_get_debug_name (node));
+          GEGL_NOTE (GEGL_DEBUG_PROCESS, "For \"%s\" processing pad '%s' result_rect = %d, %d %d×%d",
+                     gegl_pad_get_name (pad), gegl_node_get_debug_name (node),
+                     context->result_rect.x, context->result_rect.y, context->result_rect.width, context->result_rect.height);
           gegl_operation_process (operation, context, gegl_pad_get_name (pad),
                                   &context->result_rect);
-          babl_time = babl_total_usecs - babl_time;
           time      = gegl_ticks () - time;
 
           gegl_instrument ("process", gegl_node_get_operation (node), time);
-          gegl_instrument (gegl_node_get_operation (node), "babl", babl_time);
+
+          if (gegl_pad_get_num_connections (pad) > 1)
+            {
+              /* Mark buffers that have been consumed by different parts of the
+               * graph so that in-place processing can be avoided on them.
+               */
+              GValue *value;
+              GeglBuffer *buffer;
+              value = gegl_operation_context_get_value (context, gegl_pad_get_name (pad));
+              if (value)
+                {
+                  buffer = g_value_get_object (value);
+                  if (buffer)
+                    gegl_object_set_has_forked (buffer);
+                }
+            }
         }
     }
   else if (gegl_pad_is_input (pad))
