@@ -20,7 +20,8 @@
 #include <string.h>
 #include <glib-object.h>
 
-#include "gegl-types.h"
+#include "gegl.h"
+#include "gegl-types-internal.h"
 #include "gegl-finish-visitor.h"
 #include "graph/gegl-node.h"
 #include "graph/gegl-pad.h"
@@ -30,8 +31,8 @@
 
 
 static void gegl_finish_visitor_class_init (GeglFinishVisitorClass *klass);
-static void visit_node (GeglVisitor *self,
-                        GeglNode    *node);
+static void gegl_finish_visitor_visit_node (GeglVisitor            *self,
+                                            GeglNode               *node);
 
 
 G_DEFINE_TYPE (GeglFinishVisitor, gegl_finish_visitor, GEGL_TYPE_VISITOR)
@@ -42,7 +43,7 @@ gegl_finish_visitor_class_init (GeglFinishVisitorClass *klass)
 {
   GeglVisitorClass *visitor_class = GEGL_VISITOR_CLASS (klass);
 
-  visitor_class->visit_node = visit_node;
+  visitor_class->visit_node = gegl_finish_visitor_visit_node;
 }
 
 static void
@@ -51,28 +52,10 @@ gegl_finish_visitor_init (GeglFinishVisitor *self)
 }
 
 static void
-visit_node (GeglVisitor *self,
-            GeglNode    *node)
+gegl_finish_visitor_visit_node (GeglVisitor *self,
+                                GeglNode    *node)
 {
   GEGL_VISITOR_CLASS (gegl_finish_visitor_parent_class)->visit_node (self, node);
 
-  {
-    const gchar *name = gegl_node_get_name (node);
-    if (name && !strcmp (name, "proxynop-output"))
-      {
-        GeglGraph *graph = g_object_get_data (G_OBJECT (node), "graph");
-        g_assert (graph);
-        if (GEGL_NODE (graph)->operation)
-          {
-            /* issuing a finish on the graph, FIXME: we might need to do
-             * a cycle of finishs as deep as the nesting of graphs,.
-             * (or find a better way to do this) */
-
-            /* we probably do not need to recurse, prepare should have done that
-             * for us..*/
-          }
-      }
-  }
-
   gegl_node_remove_context (node, self->context_id);
 }
