@@ -118,7 +118,7 @@ gegl_processor_class_init (GeglProcessorClass *klass)
                                    g_param_spec_int ("chunksize",
                                                      "chunksize",
                                                      "Size of chunks being rendered (larger chunks need more memory to do the processing).",
-                                                     1, 1024 * 1024, gegl_config()->chunk_size, 
+                                                     1, 1024 * 1024, gegl_config()->chunk_size,
                                                      G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
 }
@@ -264,6 +264,14 @@ gegl_processor_set_node (GeglProcessor *processor,
                    GEGL_TYPE_OPERATION_SINK))
     {
       processor->input = gegl_node_get_producer (processor->node, "input", NULL);
+
+      if (processor->input == NULL)
+        {
+          g_critical ("Prepared to process a sink operation, but it "
+                      "had no \"input\" pad connected!");
+          return;
+        }
+
       if (!gegl_operation_sink_needs_full (processor->node->operation))
         {
           processor->valid_region = gegl_region_new ();
@@ -281,6 +289,8 @@ gegl_processor_set_node (GeglProcessor *processor,
       processor->valid_region = NULL;
     }
 
+  g_return_if_fail (processor->input != NULL);
+
   g_object_ref (processor->input);
 
   g_object_notify (G_OBJECT (processor), "node");
@@ -301,6 +311,8 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
   GSList        *iter;
   GeglRectangle  input_bounding_box;
 
+  g_return_if_fail (processor->input != NULL);
+
   if (! rectangle)
     {
       input_bounding_box = gegl_node_get_bounding_box (processor->input);
@@ -433,7 +445,7 @@ render_rectangle (GeglProcessor *processor)
             if (dr->width > dr->height)
               {
                 band_size = gegl_processor_get_band_size ( dr->width );
-              
+
                 fragment->width = band_size;
                 dr->width      -= band_size;
                 dr->x          += band_size;
@@ -459,7 +471,7 @@ render_rectangle (GeglProcessor *processor)
 
           return TRUE;
         }
-	
+
       if (buffered)
         {
           /* only do work if the rectangle is not completely inside the valid
@@ -563,6 +575,8 @@ gegl_processor_progress (GeglProcessor *processor)
   gint        wanted;
   gdouble     ret;
 
+  g_return_val_if_fail (processor->input != NULL, 1);
+
   if (processor->valid_region)
     {
       valid_region = processor->valid_region;
