@@ -26,17 +26,14 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-
 #include <math.h>
 #include <gegl.h>
 #include <gegl-plugin.h>
 #include <gegl-utils.h>
-#include "buffer/gegl-sampler.h"
 #include <graph/gegl-pad.h>
 #include <graph/gegl-node.h>
 #include <graph/gegl-connection.h>
 
-
 #include "affine.h"
 #include "module.h"
 
@@ -49,7 +46,7 @@ enum
   PROP_LANCZOS_WIDTH
 };
 
-
+static void          gegl_affine_finalize                  (GObject              *object);
 static void          gegl_affine_get_property              (GObject              *object,
                                                             guint                 prop_id,
                                                             GValue               *value,
@@ -64,7 +61,7 @@ static void          gegl_affine_bounding_box              (gdouble
 static gboolean      gegl_affine_is_intermediate_node      (OpAffine             *affine);
 static gboolean      gegl_affine_is_composite_node         (OpAffine             *affine);
 static void          gegl_affine_get_source_matrix         (OpAffine             *affine,
-                                                            GeglMatrix3           output);
+                                                            GeglMatrix3          *output);
 static GeglRectangle gegl_affine_get_bounding_box          (GeglOperation        *op);
 static GeglRectangle gegl_affine_get_invalidated_by_change (GeglOperation        *operation,
                                                             const gchar          *input_pad,
@@ -86,9 +83,9 @@ static GeglNode    * gegl_affine_detect                    (GeglOperation
                                                             gint                  x,
                                                             gint                  y);
 
-static gboolean      gegl_affine_matrix3_allow_fast_translate      (GeglMatrix3 matrix);
-static gboolean      gegl_affine_matrix3_allow_fast_reflect_x      (GeglMatrix3 matrix);
-static gboolean      gegl_affine_matrix3_allow_fast_reflect_y      (GeglMatrix3 matrix);
+static gboolean      gegl_affine_matrix3_allow_fast_translate      (GeglMatrix3 *matrix);
+static gboolean      gegl_affine_matrix3_allow_fast_reflect_x      (GeglMatrix3 *matrix);
+static gboolean      gegl_affine_matrix3_allow_fast_reflect_y      (GeglMatrix3 *matrix);
 
 static void          gegl_affine_fast_reflect_x            (GeglBuffer           *dest,
                                                             GeglBuffer           *src,
@@ -142,41 +139,6 @@ op_affine_get_type (void)
   return g_define_type_id;
 }
 
-
-GType
-gegl_sampler_type_from_interpolation (GeglInterpolation interpolation);
-
-/* ************************* */
-static GeglSampler *
-op_affine_sampler (OpAffine *self)
-{
-  Babl                 *format;
-  GeglSampler          *sampler;
-  GType                 desired_type;
-  GeglInterpolation     interpolation;
-
-  format = babl_format ("RaGaBaA float");
-
-  interpolation = gegl_buffer_interpolation_from_string (self->filter);
-  desired_type = gegl_sampler_type_from_interpolation (interpolation);
-
-  if (interpolation == GEGL_INTERPOLATION_LANCZOS)
-    {
-      sampler = g_object_new (desired_type,
-                              "format", format,
-                              "lanczos_width",  self->lanczos_width,
-                              NULL);
-    }
-  else
-    {
-      sampler = g_object_new (desired_type,
-                              "format", format,
-                              NULL);
-    }
-  return sampler;
-}
-/* XXX: keep a pool of samplers */
-
 #if 0
 static void
 op_affine_sampler_init (OpAffine *self)
@@ -218,6 +180,7 @@ op_affine_class_init (OpAffineClass *klass)
 
   gobject_class->set_property         = gegl_affine_set_property;
   gobject_class->get_property         = gegl_affine_get_property;
+  gobject_class->finalize             = gegl_affine_finalize;
 
   op_class->get_invalidated_by_change = gegl_affine_get_invalidated_by_change;
   op_class->get_bounding_box          = gegl_affine_get_bounding_box;
@@ -252,7 +215,7 @@ op_affine_class_init (OpAffineClass *klass)
                                    g_param_spec_string (
                                      "filter",
                                      _("Filter"),
-                                     _("Filter type (nearest, linear, lanczos, cubic, downsharp, downsize, downsmooth, upsharp, upsize, upsmooth)"),
+                                     _("Filter type (nearest, linear, lanczos, cubic, lohalo)"),
                                      "linear",
                                      G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_HARD_EDGES,
@@ -271,6 +234,12 @@ op_affine_class_init (OpAffineClass *klass)
                                      G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
 }
 
+static void
+gegl_affine_finalize (GObject *object)
+{
+  g_free (OP_AFFINE (object)->filter);
+  G_OBJECT_CLASS (op_affine_parent_class)->finalize (object);
+}
 
 static void
 op_affine_init (OpAffine *self)
@@ -326,7 +295,7 @@ gegl_affine_set_property (GObject      *object,
       break;
     case PROP_FILTER:
       g_free (self->filter);
-      self->filter = g_strdup (g_value_get_string (value));
+      self->filter = g_value_dup_string (value);
       break;
     case PROP_HARD_EDGES:
       self->hard_edges = g_value_get_boolean (value);
@@ -342,7 +311,7 @@ gegl_affine_set_property (GObject      *object,
 
 static void
 gegl_affine_create_matrix (OpAffine    *affine,
-                           GeglMatrix3  matrix)
+                           GeglMatrix3 *matrix)
 {
   gegl_matrix3_identity (matrix);
 
@@ -352,7 +321,7 @@ gegl_affine_create_matrix (OpAffine    *affine,
 
 static void
 gegl_affine_create_composite_matrix (OpAffine    *affine,
-                                     GeglMatrix3  matrix)
+                                     GeglMatrix3 *matrix)
 {
   gegl_affine_create_matrix (affine, matrix);
 
@@ -363,8 +332,8 @@ gegl_affine_create_composite_matrix (OpAffine    *affine,
     {
       GeglMatrix3 source;
 
-      gegl_affine_get_source_matrix (affine, source);
-      gegl_matrix3_multiply (source, matrix, matrix);
+      gegl_affine_get_source_matrix (affine, &source);
+      gegl_matrix3_multiply (&source, matrix, matrix);
     }
 }
 
@@ -453,7 +422,7 @@ gegl_affine_is_composite_node (OpAffine *affine)
 
 static void
 gegl_affine_get_source_matrix (OpAffine    *affine,
-                               GeglMatrix3  output)
+                               GeglMatrix3 *output)
 {
   GSList        *connections;
   GeglOperation *op = GEGL_OPERATION (affine);
@@ -483,17 +452,18 @@ gegl_affine_get_bounding_box (GeglOperation *op)
   GeglRectangle  context_rect;
   GeglSampler   *sampler;
 
-  sampler = op_affine_sampler (OP_AFFINE (op));
-  context_rect = sampler->context_rect;
+  sampler = gegl_buffer_sampler_new (NULL, babl_format("RaGaBaA float"),
+      gegl_sampler_type_from_string (affine->filter));
+  context_rect = *gegl_sampler_get_context_rect (sampler);
   g_object_unref (sampler);
 
   if (gegl_operation_source_get_bounding_box (op, "input"))
     in_rect = *gegl_operation_source_get_bounding_box (op, "input");
 
-  gegl_affine_create_composite_matrix (affine, matrix);
+  gegl_affine_create_composite_matrix (affine, &matrix);
 
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (matrix))
+      gegl_matrix3_is_identity (&matrix))
     {
       return in_rect;
     }
@@ -516,7 +486,7 @@ gegl_affine_get_bounding_box (GeglOperation *op)
   have_points [7] = in_rect.y + in_rect.height ;
 
   for (i = 0; i < 8; i += 2)
-    gegl_matrix3_transform_point (matrix,
+    gegl_matrix3_transform_point (&matrix,
                              have_points + i, have_points + i + 1);
 
   gegl_affine_bounding_box (have_points, 4, &have_rect);
@@ -535,7 +505,7 @@ gegl_affine_detect (GeglOperation *operation,
   gint         i;
 
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (inverse))
+      gegl_matrix3_is_identity (&inverse))
     {
       return gegl_operation_detect (source_node->operation, x, y);
     }
@@ -543,11 +513,11 @@ gegl_affine_detect (GeglOperation *operation,
   need_points [0] = x;
   need_points [1] = y;
 
-  gegl_affine_create_matrix (affine, inverse);
-  gegl_matrix3_invert (inverse);
+  gegl_affine_create_matrix (affine, &inverse);
+  gegl_matrix3_invert (&inverse);
 
   for (i = 0; i < 2; i += 2)
-    gegl_matrix3_transform_point (inverse,
+    gegl_matrix3_transform_point (&inverse,
                              need_points + i, need_points + i + 1);
 
   return gegl_operation_detect (source_node->operation,
@@ -569,15 +539,16 @@ gegl_affine_get_required_for_output (GeglOperation       *op,
   gint           i;
 
   requested_rect = *region;
-  sampler = op_affine_sampler (OP_AFFINE (op));
-  context_rect = sampler->context_rect;
+  sampler = gegl_buffer_sampler_new (NULL, babl_format("RaGaBaA float"),
+      gegl_sampler_type_from_string (affine->filter));
+  context_rect = *gegl_sampler_get_context_rect (sampler);
   g_object_unref (sampler);
 
-  gegl_affine_create_composite_matrix (affine, inverse);
-  gegl_matrix3_invert (inverse);
+  gegl_affine_create_composite_matrix (affine, &inverse);
+  gegl_matrix3_invert (&inverse);
 
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (inverse))
+      gegl_matrix3_is_identity (&inverse))
     {
       return requested_rect;
     }
@@ -595,7 +566,7 @@ gegl_affine_get_required_for_output (GeglOperation       *op,
   need_points [7] = requested_rect.y + requested_rect.height ;
 
   for (i = 0; i < 8; i += 2)
-    gegl_matrix3_transform_point (inverse,
+    gegl_matrix3_transform_point (&inverse,
                              need_points + i, need_points + i + 1);
   gegl_affine_bounding_box (need_points, 4, &need_rect);
 
@@ -620,25 +591,26 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
   gint               i;
   GeglRectangle      region = *input_region;
 
-  sampler = op_affine_sampler (OP_AFFINE (op));
-  context_rect = sampler->context_rect;
+  sampler = gegl_buffer_sampler_new (NULL, babl_format("RaGaBaA float"),
+      gegl_sampler_type_from_string (affine->filter));
+  context_rect = *gegl_sampler_get_context_rect (sampler);
   g_object_unref (sampler);
 
-  gegl_affine_create_matrix (affine, matrix);
+  gegl_affine_create_matrix (affine, &matrix);
 
   if (affine->origin_x || affine->origin_y)
-    gegl_matrix3_originate (matrix, affine->origin_x, affine->origin_y);
+    gegl_matrix3_originate (&matrix, affine->origin_x, affine->origin_y);
 
   if (gegl_affine_is_composite_node (affine))
     {
       GeglMatrix3 source;
 
-      gegl_affine_get_source_matrix (affine, source);
-      gegl_matrix3_multiply (source, matrix, matrix);
+      gegl_affine_get_source_matrix (affine, &source);
+      gegl_matrix3_multiply (&source, &matrix, &matrix);
     }
 
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (matrix))
+      gegl_matrix3_is_identity (&matrix))
     {
       return region;
     }
@@ -661,7 +633,7 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
   affected_points [7] = region.y + region.height ;
 
   for (i = 0; i < 8; i += 2)
-    gegl_matrix3_transform_point (matrix,
+    gegl_matrix3_transform_point (&matrix,
                              affected_points + i, affected_points + i + 1);
 
   gegl_affine_bounding_box (affected_points, 4, &affected_rect);
@@ -672,7 +644,7 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
 static void
 affine_generic (GeglBuffer  *dest,
                 GeglBuffer  *src,
-                GeglMatrix3  matrix,
+                GeglMatrix3 *matrix,
                 GeglSampler *sampler)
 {
   GeglBufferIterator *i;
@@ -707,22 +679,22 @@ affine_generic (GeglBuffer  *dest,
       GeglRectangle *roi = &i->roi[0];
       dest_buf           = (gfloat *)i->data[0];
 
-      gegl_matrix3_copy (inverse, matrix);
-      gegl_matrix3_invert (inverse);
-      inverse_jacobian[0][0] = inverse[0][0];
-      inverse_jacobian[0][1] = inverse[0][1];
-      inverse_jacobian[1][0] = inverse[1][0];
-      inverse_jacobian[1][1] = inverse[1][1];
+      gegl_matrix3_copy_into (&inverse, matrix);
+      gegl_matrix3_invert (&inverse);
+      inverse_jacobian.coeff[0][0] = inverse.coeff[0][0];
+      inverse_jacobian.coeff[0][1] = inverse.coeff[0][1];
+      inverse_jacobian.coeff[1][0] = inverse.coeff[1][0];
+      inverse_jacobian.coeff[1][1] = inverse.coeff[1][1];
 
      /* set inverse_jacobian for samplers that support it */
-      sampler->inverse_jacobian = &inverse_jacobian;
-
-      u_start = inverse[0][0] * roi->x + inverse[0][1] * roi->y + inverse[0][2];
-      v_start = inverse[1][0] * roi->x + inverse[1][1] * roi->y + inverse[1][2];
+      u_start = inverse.coeff[0][0] * roi->x + inverse.coeff[0][1]
+                    * roi->y + inverse.coeff[0][2];
+      v_start = inverse.coeff[1][0] * roi->x + inverse.coeff[1][1]
+                    * roi->y + inverse.coeff[1][2];
 
       /* correct rounding on e.g. negative scaling (is this sound?) */
-      if (inverse [0][0] < 0.)  u_start -= .001;
-      if (inverse [1][1] < 0.)  v_start -= .001;
+      if (inverse.coeff [0][0] < 0.)  u_start -= .001;
+      if (inverse.coeff [1][1] < 0.)  v_start -= .001;
 
       for (dest_ptr = dest_buf, y = roi->height; y--;)
         {
@@ -731,48 +703,48 @@ affine_generic (GeglBuffer  *dest,
 
            for (x = roi->width; x--;)
              {
-               gegl_sampler_get (sampler, u_float, v_float, dest_ptr);
+               gegl_sampler_get (sampler, u_float, v_float, &inverse_jacobian, dest_ptr);
                dest_ptr+=4;
-               u_float += inverse [0][0];
-               v_float += inverse [1][0];
+               u_float += inverse.coeff [0][0];
+               v_float += inverse.coeff [1][0];
              }
-           u_start += inverse [0][1];
-           v_start += inverse [1][1];
+           u_start += inverse.coeff [0][1];
+           v_start += inverse.coeff [1][1];
         }
     }
 }
 
 static gboolean
-gegl_affine_matrix3_allow_fast_translate (GeglMatrix3 matrix)
+gegl_affine_matrix3_allow_fast_translate (GeglMatrix3 *matrix)
 {
-  if (! GEGL_FLOAT_EQUAL (matrix[0][2], (gint) matrix[0][2]) ||
-      ! GEGL_FLOAT_EQUAL (matrix[1][2], (gint) matrix[1][2]))
+  if (! GEGL_FLOAT_EQUAL (matrix->coeff[0][2], (gint) matrix->coeff[0][2]) ||
+      ! GEGL_FLOAT_EQUAL (matrix->coeff[1][2], (gint) matrix->coeff[1][2]))
     return FALSE;
   return gegl_matrix3_is_translate (matrix);
 }
 
 static gboolean
-gegl_affine_matrix3_allow_fast_reflect_x (GeglMatrix3 matrix)
+gegl_affine_matrix3_allow_fast_reflect_x (GeglMatrix3 *matrix)
 {
   GeglMatrix3 copy;
 
-  if (! GEGL_FLOAT_EQUAL (matrix[1][1], -1.0))
+  if (! GEGL_FLOAT_EQUAL (matrix->coeff[1][1], -1.0))
     return FALSE;
-  gegl_matrix3_copy (copy, matrix);
-  copy[1][1] = 1.;
-  return gegl_affine_matrix3_allow_fast_translate (copy);
+  gegl_matrix3_copy_into (&copy, matrix);
+  copy.coeff[1][1] = 1.;
+  return gegl_affine_matrix3_allow_fast_translate (&copy);
 }
 
 static gboolean
-gegl_affine_matrix3_allow_fast_reflect_y (GeglMatrix3 matrix)
+gegl_affine_matrix3_allow_fast_reflect_y (GeglMatrix3 *matrix)
 {
   GeglMatrix3 copy;
 
-  if (! GEGL_FLOAT_EQUAL (matrix[0][0], -1.0))
+  if (! GEGL_FLOAT_EQUAL (matrix->coeff[0][0], -1.0))
     return FALSE;
-  gegl_matrix3_copy (copy, matrix);
-  copy[0][0] = 1.;
-  return gegl_affine_matrix3_allow_fast_translate (copy);
+  gegl_matrix3_copy_into (&copy, matrix);
+  copy.coeff[0][0] = 1.;
+  return gegl_affine_matrix3_allow_fast_translate (&copy);
 }
 
 static void
@@ -856,11 +828,6 @@ gegl_affine_fast_reflect_y (GeglBuffer              *dest,
   g_free (buf);
 }
 
-void  gegl_sampler_prepare     (GeglSampler *self);
-  /*XXX: Eeeek, obsessive avoidance of public headers, the API needed to
-   *     satisfy this use case should probably be provided.
-   */
-
 static gboolean
 gegl_affine_process (GeglOperation        *operation,
                      GeglOperationContext *context,
@@ -872,10 +839,10 @@ gegl_affine_process (GeglOperation        *operation,
   GeglMatrix3          matrix;
   OpAffine            *affine = (OpAffine *) operation;
 
-  gegl_affine_create_composite_matrix (affine, matrix);
+  gegl_affine_create_composite_matrix (affine, &matrix);
 
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (matrix))
+      gegl_matrix3_is_identity (&matrix))
     {
       /* passing straight through (like gegl:nop) */
       input  = gegl_operation_context_get_source (context, "input");
@@ -887,8 +854,8 @@ gegl_affine_process (GeglOperation        *operation,
 
       gegl_operation_context_take_object (context, "output", G_OBJECT (input));
     }
-  else if (gegl_affine_matrix3_allow_fast_translate (matrix) ||
-           (gegl_matrix3_is_translate (matrix) &&
+  else if (gegl_affine_matrix3_allow_fast_translate (&matrix) ||
+           (gegl_matrix3_is_translate (&matrix) &&
             ! strcmp (affine->filter, "nearest")))
     {
       /* doing a buffer shifting trick, (enhanced nop) */
@@ -896,8 +863,8 @@ gegl_affine_process (GeglOperation        *operation,
 
       output = g_object_new (GEGL_TYPE_BUFFER,
                              "source",    input,
-                             "shift-x",   (int)-matrix[0][2],
-                             "shift-y",   (int)-matrix[1][2],
+                             "shift-x",   (int)-matrix.coeff[0][2],
+                             "shift-y",   (int)-matrix.coeff[1][2],
                              "abyss-width", -1,  /* turn of abyss
                                                     (relying on abyss
                                                     of source) */
@@ -911,10 +878,11 @@ gegl_affine_process (GeglOperation        *operation,
       if (input != NULL)
         g_object_unref (input);
     }
-  else if (gegl_affine_matrix3_allow_fast_reflect_x (matrix))
+  else if (gegl_affine_matrix3_allow_fast_reflect_x (&matrix))
     {
       GeglRectangle      src_rect;
       GeglSampler       *sampler;
+      GeglRectangle      context_rect;
 
       input  = gegl_operation_context_get_source (context, "input");
       if (!input)
@@ -928,19 +896,24 @@ gegl_affine_process (GeglOperation        *operation,
       src_rect = gegl_operation_get_required_for_output (operation, "output", result);
       src_rect.y += 1;
 
-      sampler = op_affine_sampler (OP_AFFINE (operation));
-      src_rect.width -= sampler->context_rect.width;
-      src_rect.height -= sampler->context_rect.height;
+      sampler = gegl_buffer_sampler_new (input, babl_format("RaGaBaA float"),
+          gegl_sampler_type_from_string (affine->filter));
+      context_rect = *gegl_sampler_get_context_rect (sampler);
+
+      src_rect.width -= context_rect.width;
+      src_rect.height -= context_rect.height;
 
       gegl_affine_fast_reflect_x (output, input, result, &src_rect);
+      g_object_unref (sampler);
 
       if (input != NULL)
         g_object_unref (input);
     }
-  else if (gegl_affine_matrix3_allow_fast_reflect_y (matrix))
+  else if (gegl_affine_matrix3_allow_fast_reflect_y (&matrix))
     {
       GeglRectangle      src_rect;
       GeglSampler       *sampler;
+      GeglRectangle      context_rect;
 
       input  = gegl_operation_context_get_source (context, "input");
       if (!input)
@@ -954,11 +927,15 @@ gegl_affine_process (GeglOperation        *operation,
       src_rect = gegl_operation_get_required_for_output (operation, "output", result);
       src_rect.x += 1;
 
-      sampler = op_affine_sampler (OP_AFFINE (operation));
-      src_rect.width -= sampler->context_rect.width;
-      src_rect.height -= sampler->context_rect.height;
+      sampler = gegl_buffer_sampler_new (input, babl_format("RaGaBaA float"),
+          gegl_sampler_type_from_string (affine->filter));
+      context_rect = *gegl_sampler_get_context_rect (sampler);
+
+      src_rect.width -= context_rect.width;
+      src_rect.height -= context_rect.height;
 
       gegl_affine_fast_reflect_y (output, input, result, &src_rect);
+      g_object_unref (sampler);
 
       if (input != NULL)
         g_object_unref (input);
@@ -971,12 +948,9 @@ gegl_affine_process (GeglOperation        *operation,
       input  = gegl_operation_context_get_source (context, "input");
       output = gegl_operation_context_get_target (context, "output");
 
-      sampler = op_affine_sampler (affine);
-      g_object_set(sampler, "buffer", input, NULL);
-      gegl_sampler_prepare (sampler);
-      affine_generic (output, input, matrix, sampler);
-      g_object_unref(sampler->buffer);
-      sampler->buffer = NULL;
+      sampler = gegl_buffer_sampler_new (input, babl_format("RaGaBaA float"),
+          gegl_sampler_type_from_string (affine->filter));
+      affine_generic (output, input, &matrix, sampler);
       g_object_unref (sampler);
 
       if (input != NULL)
