@@ -16,6 +16,7 @@
  * Copyright 2006 Philip Lafleur
  */
 
+#include "config.h"
 #include <gegl-plugin.h>
 #include "module.h"
 #include "affine.h"
@@ -59,5 +60,8 @@ gegl_module_register (GTypeModule *module)
   dummy = reflect_get_type ();
   dummy = transform_get_type ();
 
+  dummy ++; /* silence gcc, having it is required to avoid optimizing
+               away the _get_type calls themselves */
+
   return TRUE;
 }
