@@ -36,12 +36,12 @@ gegl_chant_double (y, -G_MAXDOUBLE, G_MAXDOUBLE, 1., _("Vertical scale factor.")
 
 static void
 create_matrix (OpAffine    *op,
-               GeglMatrix3  matrix)
+               GeglMatrix3 *matrix)
 {
   GeglChantOperation *chant = GEGL_CHANT_OPERATION (op);
 
-  matrix [0][0] = chant->x;
-  matrix [1][1] = chant->y;
+  matrix->coeff [0][0] = chant->x;
+  matrix->coeff [1][1] = chant->y;
 }
 
 #endif
