@@ -38,12 +38,12 @@ gegl_chant_double (y, -G_MAXDOUBLE, G_MAXDOUBLE, 1.,
 
 static void
 create_matrix (OpAffine    *op,
-               GeglMatrix3  matrix)
+               GeglMatrix3 *matrix)
 {
   GeglChantOperation *chant = GEGL_CHANT_OPERATION (op);
 
-  matrix [0][1] = chant->x;
-  matrix [1][0] = chant->y;
+  matrix->coeff [0][1] = chant->x;
+  matrix->coeff [1][0] = chant->y;
 }
 
 #endif
