@@ -30,7 +30,7 @@ gegl_chant_double (y, -G_MAXDOUBLE, G_MAXDOUBLE, 1.,
 #else
 
 #define GEGL_CHANT_NAME translate
-#define GEGL_CHANT_DESCRIPTION  _("Repositions the buffer (with subpixel precision).")
+#define GEGL_CHANT_DESCRIPTION  _("Repositions the buffer (with subpixel precision), if integer coordinates are passed a fast-path without resampling is used.")
 #define GEGL_CHANT_SELF "translate.c"
 #include "chant.h"
 
@@ -38,12 +38,12 @@ gegl_chant_double (y, -G_MAXDOUBLE, G_MAXDOUBLE, 1.,
 
 static void
 create_matrix (OpAffine    *op,
-               GeglMatrix3  matrix)
+               GeglMatrix3 *matrix)
 {
   GeglChantOperation *chant = GEGL_CHANT_OPERATION (op);
 
-  matrix [0][2] = chant->x;
-  matrix [1][2] = chant->y;
+  matrix->coeff [0][2] = chant->x;
+  matrix->coeff [1][2] = chant->y;
 }
 
 #endif
