@@ -47,27 +47,29 @@ gegl_chant_double (rgamma, _("Radial Gamma"), 0.0, 8.0, 2.0,
 
 #define RGAMMA 2.0
 
-static void stress (GeglBuffer *src,
-                    GeglBuffer *dst,
-                    gint        radius,
-                    gint        samples,
-                    gint        iterations,
-                    gdouble     rgamma)
+static void c2g (GeglBuffer          *src,
+                 const GeglRectangle *src_rect,
+                 GeglBuffer          *dst,
+                 const GeglRectangle *dst_rect,
+                 gint                 radius,
+                 gint                 samples,
+                 gint                 iterations,
+                 gdouble              rgamma)
 {
   gint x,y;
   gint    dst_offset=0;
   gfloat *src_buf;
   gfloat *dst_buf;
-  gint    inw = gegl_buffer_get_width (src);
-  gint    outw = gegl_buffer_get_width (dst);
-  gint    inh = gegl_buffer_get_height (src);
+  gint    inw = src_rect->width;
+  gint    outw = dst_rect->width;
+  gint    inh = src_rect->height;
 
-  src_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (src) * 4);
-  dst_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (dst) * 2);
+  src_buf = g_new0 (gfloat, src_rect->width * src_rect->height * 4);
+  dst_buf = g_new0 (gfloat, dst_rect->width * dst_rect->height * 2);
 
-  gegl_buffer_get (src, 1.0, NULL, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (src, 1.0, src_rect, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
 
-  for (y=radius; y<gegl_buffer_get_height (dst)+radius; y++)
+  for (y=radius; y<dst_rect->height+radius; y++)
     {
       gint src_offset = (inw*y+radius)*4;
       for (x=radius; x<outw+radius; x++)
@@ -84,7 +86,7 @@ static void stress (GeglBuffer *src,
                              FALSE, /* same spray */
                              rgamma,
                              min, max);
-          { 
+          {
             /* this should be replaced with a better/faster projection of
              * pixel onto the vector spanned by min -> max, currently
              * computed by comparing the distance to min with the sum
@@ -104,7 +106,7 @@ static void stress (GeglBuffer *src,
             denominator = sqrt (denominator);
             denominator = nominator + denominator;
  	
-            if (denominator>0.000) 
+            if (denominator>0.000)
               {
                 dst_buf[dst_offset+0] = nominator/denominator;
               }
@@ -120,7 +122,7 @@ static void stress (GeglBuffer *src,
           }
         }
     }
-  gegl_buffer_set (dst, NULL, babl_format ("YA float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_set (dst, dst_rect, babl_format ("YA float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
   g_free (src_buf);
   g_free (dst_buf);
 }
@@ -151,12 +153,14 @@ process (GeglOperation       *operation,
          const GeglRectangle *result)
 {
   GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglRectangle compute;
+  compute = gegl_operation_get_required_for_output (operation, "input",result);
 
-  stress (input, output,
-          o->radius,
-          o->samples,
-          o->iterations,
-          /*o->rgamma*/RGAMMA);
+  c2g (input, &compute, output, result,
+       o->radius,
+       o->samples,
+       o->iterations,
+       /*o->rgamma*/RGAMMA);
 
   return  TRUE;
 }
