@@ -1271,10 +1271,8 @@ gegl_expcombine_get_required_for_output (GeglOperation       *operation,
 static void
 gegl_chant_class_init (GeglChantClass *klass)
 {
-  GObjectClass             *object_class;
   GeglOperationClass       *operation_class;
 
-  object_class    = G_OBJECT_CLASS (klass);
   operation_class = GEGL_OPERATION_CLASS (klass);
 
   operation_class->attach            = gegl_expcombine_attach;
