@@ -294,7 +294,7 @@ fattal02_calculate_defect (gfloat              *D,
                 n = (y     ==  0 ? 0 : y - 1),
                 s = (y + 1 == sy ? y : y + 1),
                 e = (x + 1 == sx ? x : x + 1);
-  
+
           _P (D, extent_d, x, y) = _P (F, extent_f, x, y) - (
                                       _P (U, extent_u, e, y) +
                                       _P (U, extent_u, w, y) +
@@ -823,10 +823,10 @@ fattal02_gaussian_blur (const gfloat        *input,
           output[x + y * width] = p / 4.0f;
         }
 
-      output[x +           0  * width] = (3 * output[x +           0  * width] +
-                                              output[x +           1  * width]) / 4.0f;
-      output[x + (height - 1) * width] = (3 * output[x + (height - 1) * width] +
-                                              output[x + (height - 2) * width]) / 4.0f;
+      output[x +           0  * width] = (3 * temp[x +           0  * width] +
+                                              temp[x +           1  * width]) / 4.0f;
+      output[x + (height - 1) * width] = (3 * temp[x + (height - 1) * width] +
+                                              temp[x + (height - 2) * width]) / 4.0f;
     }
 
   g_free (temp);
@@ -1059,6 +1059,7 @@ fattal02_find_percentiles (const gfloat *array,
 
   *min_value = sorting[(guint)(min_percent * size)];
   *max_value = sorting[(guint)(max_percent * size)];
+  g_free (sorting);
 }
 
 /********************************************************************/
@@ -1292,6 +1293,9 @@ fattal02_process (GeglOperation       *operation,
 
   gegl_buffer_set (output, result, babl_format (OUTPUT_FORMAT), pix,
                    GEGL_AUTO_ROWSTRIDE);
+  g_free (pix);
+  g_free (lum_out);
+  g_free (lum_in);
   return TRUE;
 }
 
