@@ -144,11 +144,9 @@ gegl_chant_class_init (GeglChantClass *klass)
 {
   GObjectClass             *object_class;
   GeglOperationClass       *operation_class;
-  GeglOperationSourceClass *source_class;
 
   object_class    = G_OBJECT_CLASS (klass);
   operation_class = GEGL_OPERATION_CLASS (klass);
-  source_class    = GEGL_OPERATION_SOURCE_CLASS (klass);
 
   object_class->dispose             = gegl_introspect_dispose;
 
