@@ -41,7 +41,7 @@ static void buffer_changed (GeglBuffer          *buffer,
 static GeglBuffer *ensure_buffer (GeglOperation *operation)
 {
   GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
-  GeglBuffer   *buffer = o->chant_data; 
+  GeglBuffer   *buffer = o->chant_data;
   if (buffer)
     return buffer;
   if (!buffer)
@@ -114,10 +114,8 @@ static void
 gegl_chant_class_init (GeglChantClass *klass)
 {
   GeglOperationClass       *operation_class;
-  GeglOperationSourceClass *source_class;
 
   operation_class = GEGL_OPERATION_CLASS (klass);
-  source_class    = GEGL_OPERATION_SOURCE_CLASS (klass);
 
   G_OBJECT_CLASS (klass)->dispose = dispose;
 
