@@ -142,7 +142,7 @@ process (GeglOperation       *operation,
   return TRUE;
 }
 
-static GeglRectangle 
+static GeglRectangle
 get_cached_region (GeglOperation *operation,
                    const GeglRectangle *roi)
 {
@@ -163,11 +163,9 @@ static void
 gegl_chant_class_init (GeglChantClass *klass)
 {
   GeglOperationClass       *operation_class;
-  GeglOperationSourceClass *source_class;
   GObjectClass             *object_class;
 
   operation_class = GEGL_OPERATION_CLASS (klass);
-  source_class    = GEGL_OPERATION_SOURCE_CLASS (klass);
   object_class    = G_OBJECT_CLASS (klass);
 
   object_class->finalize = finalize;
