@@ -120,15 +120,13 @@ prepare (GeglOperation *operation)
 static void
 gegl_chant_class_init (GeglChantClass *klass)
 {
-  GObjectClass       *object_class;
   GeglOperationClass *operation_class;
 
-  object_class    = G_OBJECT_CLASS (klass);
   operation_class = GEGL_OPERATION_CLASS (klass);
 
   operation_class->name        = "gegl:rectangle";
   operation_class->categories  = "input";
-  operation_class->description = 
+  operation_class->description =
         _("A rectangular source of a fixed size with a solid color");
   operation_class->attach = attach;
   operation_class->prepare = prepare;
