@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (c) 2010 Mukund Sivaraman <muks@banu.com>
+ * Copyright (c) 2010, 2011 Mukund Sivaraman <muks@banu.com>
  */
 
 #include "config.h"
@@ -59,59 +59,59 @@ query_jp2 (const gchar   *path,
     {
       in = jas_stream_fopen (path, "rb");
       if (!in)
-	{
-	  g_warning ("Unable to open image file '%s'", path);
-	  break;
-	}
+        {
+          g_warning ("Unable to open image file '%s'", path);
+          break;
+        }
 
       image_fmt = jas_image_getfmt (in);
       if (image_fmt < 0)
-	{
-	  g_warning (_("Unknown JPEG-2000 image format in '%s'"), path);
+        {
+          g_warning (_("Unknown JPEG-2000 image format in '%s'"), path);
           break;
-	}
+        }
 
       image = jas_image_decode (in, image_fmt, NULL);
       if (!image)
-	{
-	  g_warning (_("Unable to open JPEG-2000 image in '%s'"), path);
-	  break;
-	}
+        {
+          g_warning (_("Unable to open JPEG-2000 image in '%s'"), path);
+          break;
+        }
 
       output_profile = jas_cmprof_createfromclrspc (JAS_CLRSPC_SRGB);
       if (!output_profile)
         {
-	  g_warning (_("Unable to create output color profile for '%s'"), path);
-	  break;
+          g_warning (_("Unable to create output color profile for '%s'"), path);
+          break;
         }
 
       cimage = jas_image_chclrspc (image, output_profile,
                                    JAS_CMXFORM_INTENT_PER);
       if (!cimage)
         {
-	  g_warning (_("Unable to convert image to sRGB color space "
+          g_warning (_("Unable to convert image to sRGB color space "
                        "when processing '%s'"), path);
-	  break;
+          break;
         }
 
       numcmpts = jas_image_numcmpts (cimage);
       if (numcmpts != 3)
-	{
-	  g_warning (_("Unsupported non-RGB JPEG-2000 file with "
+        {
+          g_warning (_("Unsupported non-RGB JPEG-2000 file with "
                        "%d components in '%s'"), numcmpts, path);
-	  break;
-	}
+          break;
+        }
 
       *width = jas_image_cmptwidth (cimage, 0);
       *height = jas_image_cmptheight (cimage, 0);
       *depth = jas_image_cmptprec (cimage, 0);
 
       if ((*depth != 8) && (*depth != 16))
-	{
-	  g_warning (_("Unsupported JPEG-2000 file with depth %d in '%s'"),
+        {
+          g_warning (_("Unsupported JPEG-2000 file with depth %d in '%s'"),
                      *depth, path);
-	  break;
-	}
+          break;
+        }
 
       b = FALSE;
 
@@ -173,58 +173,24 @@ process (GeglOperation       *operation,
   GeglRectangle rect = {0,0,0,0};
   jas_image_t *image;
   gint width, height, depth;
-  gsize bpc;
-  guchar *data = NULL;
+  guchar *data_b;
+  gushort *data_s;
   gboolean ret;
   int components[3];
   jas_matrix_t *matrices[3] = {NULL, NULL, NULL};
   gint i;
   gint row;
   gboolean b;
-  gushort *ptr_s;
-  guchar *ptr_b;
 
   image = NULL;
+  data_b = NULL;
+  data_s = NULL;
+
   width = height = depth = 0;
 
   if (!query_jp2 (o->path, &width, &height, &depth, &image))
     return FALSE;
 
-  rect.height = height;
-  rect.width = width;
-
-  switch (depth)
-    {
-    case 8:
-      bpc = sizeof (guchar);
-      break;
-
-    case 16:
-      bpc = sizeof (gushort);
-      break;
-
-    default:
-      g_warning ("%s: Programmer stupidity error", G_STRLOC);
-      return FALSE;
-    }
-
-  data = (guchar *) g_malloc (width * height * 3 * bpc);
-  ptr_s = (gushort *) data;
-  ptr_b = data;
-
-  switch (depth)
-    {
-    case 16:
-      gegl_buffer_get (output, 1.0, &rect, babl_format ("R'G'B' u16"), data,
-                       GEGL_AUTO_ROWSTRIDE);
-      break;
-
-    case 8:
-    default:
-      gegl_buffer_get (output, 1.0, &rect, babl_format ("R'G'B' u8"), data,
-                       GEGL_AUTO_ROWSTRIDE);
-    }
-
   ret = FALSE;
   b = FALSE;
 
@@ -256,6 +222,21 @@ process (GeglOperation       *operation,
       for (i = 0; i < 3; i++)
         matrices[i] = jas_matrix_create(1, width);
 
+      switch (depth)
+        {
+        case 16:
+          data_s = (gushort *) g_malloc (width * 3 * sizeof (gushort));
+          break;
+
+        case 8:
+          data_b = (guchar *) g_malloc (width * 3 * sizeof (guchar));
+          break;
+
+        default:
+          g_warning ("%s: Programmer stupidity error", G_STRLOC);
+          return FALSE;
+        }
+
       for (row = 0; row < height; row++)
         {
           gint plane, col;
@@ -280,41 +261,60 @@ process (GeglOperation       *operation,
           for (plane = 0; plane < 3; plane++)
             jrow[plane] = jas_matrix_getref (matrices[plane], 0, 0);
 
-          for (col = 0; col < width; col++)
+          switch (depth)
             {
-              switch (depth)
+            case 16:
+              for (col = 0; col < width; col++)
                 {
-                case 16:
-                  *ptr_s++ = (gushort) jrow[0][col];
-                  *ptr_s++ = (gushort) jrow[1][col];
-                  *ptr_s++ = (gushort) jrow[2][col];
-                  break;
-
-                case 8:
-                default:
-                  *ptr_b++ = (guchar) jrow[0][col];
-                  *ptr_b++ = (guchar) jrow[1][col];
-                  *ptr_b++ = (guchar) jrow[2][col];
+                  data_s[col * 3]     = (gushort) jrow[0][col];
+                  data_s[col * 3 + 1] = (gushort) jrow[1][col];
+                  data_s[col * 3 + 2] = (gushort) jrow[2][col];
                 }
+              break;
+
+            case 8:
+              for (col = 0; col < width; col++)
+                {
+                  data_b[col * 3]     = (guchar) jrow[0][col];
+                  data_b[col * 3 + 1] = (guchar) jrow[1][col];
+                  data_b[col * 3 + 2] = (guchar) jrow[2][col];
+                }
+              break;
+
+            default:
+              g_warning ("%s: Programmer stupidity error", G_STRLOC);
+              b = TRUE;
+            }
+
+          if (b)
+            break;
+
+          rect.x = 0;
+          rect.y = row;
+          rect.width = width;
+          rect.height = 1;
+
+          switch (depth)
+            {
+            case 16:
+              gegl_buffer_set (output, &rect, babl_format ("R'G'B' u16"),
+                               data_s, GEGL_AUTO_ROWSTRIDE);
+              break;
+
+            case 8:
+              gegl_buffer_set (output, &rect, babl_format ("R'G'B' u8"),
+                               data_b, GEGL_AUTO_ROWSTRIDE);
+	      break;
+
+            default:
+              g_warning ("%s: Programmer stupidity error", G_STRLOC);
+              b = TRUE;
             }
         }
 
       if (b)
         break;
 
-      switch (depth)
-        {
-        case 16:
-          gegl_buffer_set (output, &rect, babl_format ("R'G'B' u16"), data,
-                           GEGL_AUTO_ROWSTRIDE);
-          break;
-
-        case 8:
-        default:
-          gegl_buffer_set (output, &rect, babl_format ("R'G'B' u8"), data,
-                           GEGL_AUTO_ROWSTRIDE);
-        }
-
       ret = TRUE;
     }
   while (FALSE); /* structured goto */
@@ -323,8 +323,11 @@ process (GeglOperation       *operation,
     if (matrices[i])
       jas_matrix_destroy (matrices[i]);
 
-  if (data)
-    g_free (data);
+  if (data_b)
+    g_free (data_b);
+
+  if (data_s)
+    g_free (data_s);
 
   if (image)
     jas_image_destroy (image);
@@ -351,12 +354,12 @@ get_bounding_box (GeglOperation * operation)
     {
     case 16:
       gegl_operation_set_format (operation, "output",
-				 babl_format ("R'G'B' u16"));
+                                 babl_format ("R'G'B' u16"));
       break;
 
     case 8:
       gegl_operation_set_format (operation, "output",
-				 babl_format ("R'G'B' u8"));
+                                 babl_format ("R'G'B' u8"));
       break;
 
     default:
