@@ -137,9 +137,9 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->prepare = prepare;
   operation_class->get_bounding_box = get_bounding_box;
 
-  operation_class->name        = "mandelbrot";
+  operation_class->name        = "gegl:mandelbrot";
   operation_class->categories  = "render";
-  operation_class->description = _("Mandelbrot renderer.");
+  operation_class->description = _("Mandelbrot set renderer.");
 }
 
 #endif
