@@ -4,7 +4,7 @@
 reference_dir=$REFERENCE_DIR
 
 total_tests=`ls -1 $reference_dir/*.buf | wc -l`
-successful_tests=`cat buffer-tests-report | grep identical | wc -l`
+successful_tests=`cat buffer-tests-report | grep -c identical`
 
 echo $successful_tests of $total_tests tests succesful.
 
@@ -13,5 +13,5 @@ then
 exit 0;
 else
 echo Look in the file \"buffer-tests-report\" for detailed failure information.
-exit -1;
+exit 1;
 fi
