@@ -461,10 +461,10 @@ stuff (gint    argc,
                     NULL);
       GeglNode *crop = gegl_node_new_child (gegl,
        "operation", "gegl:crop",
-       "x", 0,
-       "y", 0,
-       "width", 50,
-       "height", 50,
+       "x", 0.0,
+       "y", 0.0,
+       "width", 50.0,
+       "height", 50.0,
        NULL);
       GeglNode  *png_load = gegl_node_new_child (gegl,
                     "operation", "gegl:checkerboard",
