@@ -102,52 +102,81 @@ list_properties (GType    type,
 
           if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_DOUBLE))
             {
+              gdouble default_value = G_PARAM_SPEC_DOUBLE (self[prop_no])->default_value;
+              gdouble min = G_PARAM_SPEC_DOUBLE (self[prop_no])->minimum;
+              gdouble max = G_PARAM_SPEC_DOUBLE (self[prop_no])->maximum;
 
-            g_print ("%2.2f", G_PARAM_SPEC_DOUBLE (self[prop_no])->default_value);
+              if (default_value<-10000000)
+                g_print ("-inf ");
+              else if (default_value>10000000)
+                g_print ("+inf");
+              else
+                g_print ("%2.2f", default_value);
 
-            {
-            gdouble min = G_PARAM_SPEC_DOUBLE (self[prop_no])->minimum;
-            gdouble max = G_PARAM_SPEC_DOUBLE (self[prop_no])->maximum;
-            g_print ("<br/>");
-            if (min<-10000000)
-              g_print ("-inf ");
-            else
-              g_print ("%2.2f", min);
-            
-            g_print ("-");
+              g_print ("<br/>");
+              if (min<-10000000)
+                g_print ("-inf ");
+              else
+                g_print ("%2.2f", min);
 
-            if (max>10000000)
-              g_print (" +inf");
-            else
-              g_print ("%2.2f", max);
-            }
+              g_print ("-");
 
+              if (max>10000000)
+                g_print (" +inf");
+              else
+                g_print ("%2.2f", max);
             }
           else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_INT))
             {
-              g_print ("%i", G_PARAM_SPEC_INT (self[prop_no])->default_value);
+              gint default_value = G_PARAM_SPEC_INT (self[prop_no])->default_value;
+              gint min = G_PARAM_SPEC_INT (self[prop_no])->minimum;
+              gint max = G_PARAM_SPEC_INT (self[prop_no])->maximum;
 
-            {
-            gint min = G_PARAM_SPEC_INT (self[prop_no])->minimum;
-            gint max = G_PARAM_SPEC_INT (self[prop_no])->maximum;
-            g_print ("<br/>");
-            if (min<-10000000)
-              g_print ("-inf ");
-            else
-              g_print ("%i", min);
-            
-            g_print ("-");
+              if (default_value<-10000000)
+                g_print ("-inf ");
+              else if (default_value>10000000)
+                g_print ("+inf");
+              else
+                g_print ("%i", default_value);
 
-            if (max>10000000)
-              g_print (" +inf");
-            else
-              g_print ("%i", max);
-            }
+              g_print ("<br/>");
+              if (min<-10000000)
+                g_print ("-inf ");
+              else
+                g_print ("%i", min);
 
+              g_print ("-");
+
+              if (max>10000000)
+                g_print (" +inf");
+              else
+                g_print ("%i", max);
             }
           else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_FLOAT))
             {
-              g_print ("%2.2f", G_PARAM_SPEC_FLOAT (self[prop_no])->default_value);
+              gfloat default_value = G_PARAM_SPEC_FLOAT (self[prop_no])->default_value;
+              gfloat min = G_PARAM_SPEC_FLOAT (self[prop_no])->minimum;
+              gfloat max = G_PARAM_SPEC_FLOAT (self[prop_no])->maximum;
+
+              if (default_value<-10000000)
+                g_print ("-inf ");
+              else if (default_value>10000000)
+                g_print ("+inf");
+              else
+                g_print ("%2.2f", default_value);
+
+              g_print ("<br/>");
+              if (min<-10000000)
+                g_print ("-inf ");
+              else
+                g_print ("%2.2f", min);
+
+              g_print ("-");
+
+              if (max>10000000)
+                g_print (" +inf");
+              else
+                g_print ("%2.2f", max);
             }
           else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_BOOLEAN))
             {
@@ -208,7 +237,7 @@ list_properties (GType    type,
 
 static gchar *html_top = "<html>\n<head>\n<title>GEGL operations</title>\n<link rel='shortcut icon' href='images/gegl.ico'/>\n<style type='text/css'>\n@import url(gegl.css);\ndiv#toc ul { font-size:70%; }\n"
 ".category { margin-bottom: 2em; }\n"
-".category a {\n  display: block;\n  width: 10em;\n  height: 1.2em;\n  float: left;\n  text-align: left;\n  font-size: 90%;\n}\n"
+".category a {\n  display: block;\n  width: 14em;\n  height: 1.2em;\n  float: left;\n  text-align: left;\n  font-size: 90%;\n}\n"
 "</style>\n</head>\n\n<body>\n<div class='paper'>\n<div class='content'>\n";
 static gchar *html_bottom = "</div>\n</div>\n</body>\n</html>\n";
 
@@ -245,17 +274,15 @@ static void category_menu_index (gpointer key,
   gchar    *category = key;
   GList    *operations = value;
   GList    *iter;
-  gboolean  comma;
 
   if (!strcmp (category, "hidden"))
     return;
-  for (iter=operations, comma=FALSE;iter;iter = g_list_next (iter))
+  for (iter=operations;iter;iter = g_list_next (iter))
     {
       GeglOperationClass *klass = iter->data;
       if (strstr (klass->categories, "hidden"))
         continue;
       g_print ("<li><a href='#op_%s'>%s</a></li>\n", klass->name, klass->name);
-      comma = TRUE;
     }
 }
 
@@ -266,7 +293,6 @@ main (gint    argc,
   GList      *operations;
   GList      *iter;
   GHashTable *categories = NULL;
-  gboolean    comma;
 
   gegl_init (&argc, &argv);
 
@@ -274,7 +300,7 @@ main (gint    argc,
 
   /* Collect categories */
   categories = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-  for (iter=operations, comma=FALSE;iter;iter = g_list_next (iter))
+  for (iter=operations;iter;iter = g_list_next (iter))
     {
       GeglOperationClass *klass = iter->data;
       const gchar *ptr = klass->categories;
